import time

from ddt import data, ddt, unpack
from mock import patch
from selenium.common.exceptions import NoSuchElementException
from selenium.webdriver.support.ui import WebDriverWait

from workbench.runtime import WorkbenchRuntime

from .base_test import (CORRECT, INCORRECT, PARTIAL, GetChoices,
                        MentoringAssessmentBaseTest)
from .test_dashboard import MockSubmissionsAPI


class HTMLColors(object):
    GREEN = 'rgb(0, 128, 0)'
    BLUE = 'rgb(0, 0, 255)'
    RED = 'rgb(255, 0, 0)'
    GREY = 'rgb(237, 237, 237)'
    PURPLE = 'rgb(128, 0, 128)'
    ORANGE = 'rgb(255, 165, 0)'
    CORAL = 'rgb(255, 127, 80)'
    CORNFLOWERBLUE = 'rgb(100, 149, 237)'
    OLIVE = 'rgb(128, 128, 0)'
    CRIMSON = 'rgb(220, 20, 60)'


class ExtendedMockSubmissionsAPI(MockSubmissionsAPI):
    def get_all_submissions(self, course_key_str, block_id, block_type):
        return (
            submission for submission in self.submissions.values() if
            submission['student_item']['item_id'] == block_id
        )


class MultipleSliderBlocksTestMixins():
    """ Mixins for testing slider blocks. Allows multiple slider blocks on the page. """

    def get_slider_value(self, slider_number):
        print('SLIDER NUMBER: {}'.format(slider_number))
        return int(
            self.browser.execute_script("return $('.pb-slider-range').eq(arguments[0]).val()", slider_number-1)
        )

    def set_slider_value(self, slider_number, val):
        self.browser.execute_script(
            "$('.pb-slider-range').eq(arguments[0]).val(arguments[1]).change()", slider_number-1, val
        )


@ddt
class StepBuilderTest(MentoringAssessmentBaseTest, MultipleSliderBlocksTestMixins):

    def setUp(self):
        super(StepBuilderTest, self).setUp()

        mock_submissions_api = ExtendedMockSubmissionsAPI()
        patches = (
            (
                "problem_builder.plot.PlotBlock.course_key_str",
                property(lambda block: "course_id")
            ),
            (
                "problem_builder.plot.sub_api",
                mock_submissions_api
            ),
            (
                "problem_builder.mcq.sub_api",
                mock_submissions_api
            ),
            (
                "problem_builder.slider.sub_api",
                mock_submissions_api
            ),
            (
                "problem_builder.sub_api.SubmittingXBlockMixin.student_item_key",
                property(
                    lambda block: dict(
                        student_id="student_id",
                        course_id="course_id",
                        item_id=block.scope_ids.usage_id,
                        item_type=block.scope_ids.block_type
                    )
                )
            ),
        )
        for p in patches:
            patcher = patch(*p)
            patcher.start()
            self.addCleanup(patcher.stop)

        runtime_patcher = patch(
            'workbench.runtime.WorkbenchRuntime.anonymous_student_id',
            property(lambda runtime: "student_id"),
            create=True
        )
        runtime_patcher.start()
        self.addCleanup(runtime_patcher.stop)

        # Mock replace_urls so that we can check that message HTML gets processed with any
        # transforms that the runtime needs.
        runtime_patcher2 = patch(
            'workbench.runtime.WorkbenchRuntime.replace_urls',
            lambda _runtime, html: html.replace('REPLACE-ME', ''),
            create=True
        )
        runtime_patcher2.start()
        self.addCleanup(runtime_patcher2.stop)

    def freeform_answer(
            self, number, step_builder, controls, text_input, result, saved_value="", hold=False, last=False
    ):
        self.expect_question_visible(number, step_builder)

        answer = step_builder.find_element_by_css_selector("textarea.answer.editable")

        self.assertIn(self.question_text(number), step_builder.text)
        self.assertIn("What is your goal?", step_builder.text)

        self.assertEquals(saved_value, answer.get_attribute("value"))
        if not saved_value:
            self.assert_disabled(controls.submit)

        if last:
            self.assert_disabled(controls.review)
        else:
            self.assert_disabled(controls.next_question)

        answer.clear()
        answer.send_keys(text_input)
        self.assertEquals(text_input, answer.get_attribute("value"))

        self.assert_clickable(controls.submit)
        self.ending_controls(controls, last)
        if not last:
            self.assert_hidden(controls.review)

        self.assert_hidden(controls.try_again)

        controls.submit.click()

        self.do_submit_wait(controls, last)
        self._assert_checkmark(step_builder, result)
        if not hold:
            self.do_post(controls, last)

    def single_choice_question(self, number, step_builder, controls, choice_name, result, last=False):
        question = self.expect_question_visible(number, step_builder)

        self.assertIn("Do you like this MCQ?", question.text)

        self.assert_disabled(controls.submit)
        self.ending_controls(controls, last)
        self.assert_hidden(controls.try_again)

        choices = GetChoices(question)
        expected_state = {"Yes": False, "Maybe not": False, "I don't understand": False}
        self.assertEquals(choices.state, expected_state)

        choices.select(choice_name)
        expected_state[choice_name] = True
        self.assertEquals(choices.state, expected_state)

        self.selected_controls(controls, last)

        controls.submit.click()

        self.do_submit_wait(controls, last)
        self._assert_checkmark(step_builder, result)

        self.do_post(controls, last)

    def rating_question(self, number, step_builder, controls, choice_name, result, last=False):
        self.expect_question_visible(number, step_builder)

        self.assertIn("How much do you rate this MCQ?", step_builder.text)

        self.assert_disabled(controls.submit)
        self.ending_controls(controls, last)
        self.assert_hidden(controls.try_again)

        choices = GetChoices(step_builder, ".rating")
        expected_choices = {
            "1 - Not good at all": False,
            "2": False, "3": False, "4": False,
            "5 - Extremely good": False,
            "I don't want to rate it": False,
        }
        self.assertEquals(choices.state, expected_choices)
        choices.select(choice_name)
        expected_choices[choice_name] = True
        self.assertEquals(choices.state, expected_choices)

        self.ending_controls(controls, last)

        controls.submit.click()

        self.do_submit_wait(controls, last)
        self._assert_checkmark(step_builder, result)
        self.do_post(controls, last)

    def peek_at_multiple_response_question(
        self, number, step_builder, controls, last=False, extended_feedback=False, alternative_review=False
    ):
        question = self.expect_question_visible(number, step_builder)
        self.assertIn("What do you like in this MRQ?", step_builder.text)
        return question

        if extended_feedback:
            self.assert_disabled(controls.submit)
            self.check_question_feedback(step_builder, question)
            if alternative_review:
                self.assert_clickable(controls.review_link)
                self.assert_hidden(controls.try_again)

    def assert_review_conditional_messages_equal(self, step_builder, messages_expected):
        """ Test that the Conditional Messages seen on the review match messages_expected. """
        messages = step_builder.find_elements_by_css_selector('.review-conditional-message')
        self.assertListEqual([msg.text for msg in messages], messages_expected)

    def peek_at_review(self, step_builder, controls, expected, extended_feedback=False):
        self.wait_until_text_in("You scored {percentage}% on this assessment.".format(**expected), step_builder)

        # Check grade breakdown
        if expected["correct"] == 1:
            self.assertIn("You answered 1 question correctly.".format(**expected), step_builder.text)
        else:
            self.assertIn("You answered {correct} questions correctly.".format(**expected), step_builder.text)

        if expected["partial"] == 1:
            self.assertIn("You answered 1 question partially correctly.", step_builder.text)
        else:
            self.assertIn("You answered {partial} questions partially correctly.".format(**expected), step_builder.text)

        if expected["incorrect"] == 1:
            self.assertIn("You answered 1 question incorrectly.", step_builder.text)
        else:
            self.assertIn("You answered {incorrect} questions incorrectly.".format(**expected), step_builder.text)

        # Check presence of review links
        # - If unlimited attempts: no review links, no explanation
        # - If limited attempts:
        #   - If not max attempts reached: no review links, no explanation
        #   - If max attempts reached:
        #     - If extended feedback: review links, explanation
        #     - If not extended feedback: no review links, no explanation

        review_list = step_builder.find_elements_by_css_selector('.review-list')
        try:
            step_builder.find_element_by_css_selector('.review-links-explanation')
        except NoSuchElementException:
            review_links_explained = False
        else:
            review_links_explained = True

        if expected["max_attempts"] == 0:
            self.assertFalse(review_list)
            self.assertFalse(review_links_explained)
        else:
            if expected["num_attempts"] < expected["max_attempts"]:
                self.assertFalse(review_list)
                self.assertFalse(review_links_explained)
            elif expected["num_attempts"] == expected["max_attempts"]:
                if extended_feedback:
                    for correctness in ['correct', 'incorrect', 'partial']:
                        review_items = step_builder.find_elements_by_css_selector('.%s-list li' % correctness)
                        self.assertEqual(len(review_items), expected[correctness])
                    self.assertTrue(review_links_explained)
                else:
                    self.assertFalse(review_list)
                    self.assertFalse(review_links_explained)

        # Check if info about number of attempts used is correct
        if expected["max_attempts"] == 1:
            self.assertIn("You have used {num_attempts} of 1 submission.".format(**expected), step_builder.text)
        elif expected["max_attempts"] == 0:
            self.assertNotIn("You have used", step_builder.text)
        else:
            self.assertIn(
                "You have used {num_attempts} of {max_attempts} submissions.".format(**expected),
                step_builder.text
            )

        # Check controls
        self.assert_hidden(controls.submit)
        self.assert_hidden(controls.next_question)
        self.assert_hidden(controls.review)
        self.assert_hidden(controls.review_link)

    def popup_check(self, step_builder, item_feedbacks, prefix='', do_submit=True):
        for index, expected_feedback in enumerate(item_feedbacks):

            self.wait_until_exists(prefix + " .choice")
            self.wait_until_exists(prefix + " .choice .choice-label")
            choice_wrapper = step_builder.find_elements_by_css_selector(prefix + " .choice")[index]
            choice_label = step_builder.find_elements_by_css_selector(prefix + " .choice .choice-label")[index]
            choice_label.click()

            self.wait_until_exists(".choice-result")
            item_feedback_icon = choice_wrapper.find_element_by_css_selector(".choice-result")
            item_feedback_icon.click()

            self.wait_until_exists(".choice-tips")
            item_feedback_popup = choice_wrapper.find_element_by_css_selector(".choice-tips")
            self.assertTrue(item_feedback_popup.is_displayed())
            self.assertEqual(item_feedback_popup.text, expected_feedback)

            item_feedback_popup.click()
            self.assertTrue(item_feedback_popup.is_displayed())

            step_builder.find_elements_by_css_selector(prefix + ' .question-title')[0].click()
            self.wait_until_hidden(item_feedback_popup)
            self.assertFalse(item_feedback_popup.is_displayed())

    def extended_feedback_checks(self, step_builder, controls, expected_results):
        # MRQ is third correctly answered question
        self.assert_hidden(controls.review_link)
        step_builder.find_elements_by_css_selector('.correct-list li a')[2].click()
        self.peek_at_multiple_response_question(
            None, step_builder, controls, extended_feedback=True, alternative_review=True
        )

        # Step should display 5 checkmarks (4 correct items for MRQ, plus step-level feedback about correctness)
        correct_marks = step_builder.find_elements_by_css_selector('.sb-step .checkmark-correct')
        incorrect_marks = step_builder.find_elements_by_css_selector('.sb-step .checkmark-incorrect')
        overall_mark = step_builder.find_elements_by_css_selector('.submit .checkmark-correct')
        self.assertEqual(len(correct_marks), 4)
        self.assertEqual(len(incorrect_marks), 0)
        self.assertEqual(len(overall_mark), 1)

        item_feedbacks = [
            "This is something everyone has to like about this MRQ",
            "This is something everyone has to like about this MRQ",
            "This MRQ is indeed very graceful",
            "Nah, there aren't any!"
        ]
        self.popup_check(step_builder, item_feedbacks, prefix='div[data-name="mrq_1_1"]', do_submit=False)
        controls.review_link.click()
        self.peek_at_review(step_builder, controls, expected_results, extended_feedback=True)

        # Review rating question (directly precedes MRQ)
        step_builder.find_elements_by_css_selector('.incorrect-list li a')[0].click()
        # It should be possible to visit the MRQ from here
        self.wait_until_clickable(controls.next_question)
        controls.next_question.click()
        self.html_section(step_builder, controls)
        self.peek_at_multiple_response_question(
            None, step_builder, controls, extended_feedback=True, alternative_review=True
        )

    def test_next_label(self):
        step_builder, controls = self.load_assessment_scenario("step_builder_next.xml")
        self.expect_question_visible(None, step_builder)
        self.assertEqual(controls.next_question.get_attribute('value'), "Next Challenge")
        self.freeform_answer(None, step_builder, controls, 'This is the answer', CORRECT)
        self.expect_question_visible(None, step_builder)
        self.assertEqual(controls.next_question.get_attribute('value'), "Next Item")
        self.single_choice_question(None, step_builder, controls, 'Maybe not', INCORRECT)
        self.rating_question(None, step_builder, controls, "5 - Extremely good", CORRECT, last=True)

        # Check extended feedback loads the labels correctly.
        step_builder.find_elements_by_css_selector('.correct-list li a')[0].click()
        self.expect_question_visible(None, step_builder)
        self.assertEqual(controls.next_question.get_attribute('value'), "Next Challenge")

    def html_section(self, step_builder, controls, last=False):
        self.wait_until_hidden(controls.submit)
        target_control = controls.review if last else controls.next_question
        self.wait_until_clickable(target_control)
        target_control.click()

    def test_html_last(self):
        step_builder, controls = self.load_assessment_scenario("step_builder_html_last.xml")
        # Step 1
        # Submit free-form answer, go to next step
        self.freeform_answer(None, step_builder, controls, 'This is the answer', CORRECT)

        # Step 2
        # Submit MCQ, go to next step
        self.single_choice_question(None, step_builder, controls, 'Maybe not', INCORRECT)

        self.html_section(step_builder, controls, last=True)

    @data(
        {"max_attempts": 0, "extended_feedback": False},  # Unlimited attempts, no extended feedback
        {"max_attempts": 1, "extended_feedback": True},  # Limited attempts, extended feedback
        {"max_attempts": 1, "extended_feedback": False},  # Limited attempts, no extended feedback
        {"max_attempts": 2, "extended_feedback": True},  # Limited attempts, extended feedback
    )
    def test_step_builder(self, params):
        max_attempts = params['max_attempts']
        extended_feedback = params['extended_feedback']
        step_builder, controls = self.load_assessment_scenario("step_builder.xml", params)

        # Step 1
        # Submit free-form answer, go to next step
        self.freeform_answer(None, step_builder, controls, 'This is the answer', CORRECT)

        # Step 2
        # Submit MCQ, go to next step
        self.single_choice_question(None, step_builder, controls, 'Maybe not', INCORRECT)

        # Step 3
        # Submit rating, go to next step
        self.rating_question(None, step_builder, controls, "5 - Extremely good", CORRECT)

        # Step 4, html with no question.
        self.html_section(step_builder, controls)

        # Last step
        # Submit MRQ, go to review
        with patch.object(WorkbenchRuntime, 'publish') as patched_method:
            self.multiple_response_question(None, step_builder, controls, ("Its beauty",), PARTIAL, last=True)

            # Check if "grade" event was published
            # Note that we can't use patched_method.assert_called_once_with here
            # because there is no way to obtain a reference to the block instance generated from the XML scenario
            self.assertTrue(patched_method.called)
            self.assertEquals(len(patched_method.call_args_list), 2)

            block_object = self.load_root_xblock()

            positional_args = patched_method.call_args_list[0][0]
            block, event, data = positional_args

            self.assertEquals(block.scope_ids.usage_id, block_object.scope_ids.usage_id)
            self.assertEquals(event, 'grade')
            self.assertEquals(data, {'value': 0.625, 'max_value': 1})

            # test xblock.interaction is submitted
            block, event, data = patched_method.call_args_list[1][0]
            self.assertEquals(block.scope_ids.usage_id, block_object.scope_ids.usage_id)
            self.assertEquals(event, 'xblock.interaction')
            self.assertEquals(data, {
                'attempts_max': max_attempts if max_attempts else 'unlimited', 'attempts_count': 1, 'score': 63
            })

        # Review step
        expected_results = {
            "correct": 2, "partial": 1, "incorrect": 1, "percentage": 63,
            "num_attempts": 1, "max_attempts": max_attempts
        }
        self.peek_at_review(step_builder, controls, expected_results, extended_feedback=extended_feedback)
        if extended_feedback and max_attempts == 1:
            self.assertIn("Question 1 and Question 3", step_builder.find_element_by_css_selector('.correct-list').text)

        if max_attempts == 1:
            self.assert_review_conditional_messages_equal(
                step_builder,
                ["Not quite!", "This message is shown when you run out of attempts."]
            )
            self.assert_disabled(controls.try_again)
            return

        self.assert_review_conditional_messages_equal(step_builder, ["Not quite! You can try again, though."])
        self.assert_clickable(controls.try_again)

        # Try again
        controls.try_again.click()

        self.wait_until_hidden(controls.try_again)

        # Step 1
        # Submit free-form answer, go to next step
        self.freeform_answer(
            None, step_builder, controls, 'This is a different answer', CORRECT, saved_value='This is the answer'
        )
        # Step 2
        # Reload the page, which should have no effect
        self.browser.execute_script("$(document).html(' ');")
        step_builder, controls = self.go_to_assessment()
        # Submit MCQ, go to next step
        self.single_choice_question(None, step_builder, controls, 'Yes', CORRECT)
        # Step 3
        # Submit rating, go to next step
        self.rating_question(None, step_builder, controls, "1 - Not good at all", INCORRECT)

        # Step 4, read only. Go to next step.
        self.html_section(step_builder, controls)
        # Last step
        # Submit MRQ, go to review
        user_selection = ("Its elegance", "Its beauty", "Its gracefulness")
        self.multiple_response_question(None, step_builder, controls, user_selection, CORRECT, last=True)

        # Review step
        expected_results = {
            "correct": 3, "partial": 0, "incorrect": 1, "percentage": 75,
            "num_attempts": 2, "max_attempts": max_attempts
        }
        self.peek_at_review(step_builder, controls, expected_results, extended_feedback=extended_feedback)
        if extended_feedback and max_attempts == 2:
            self.assertIn(
                "Question 1, Question 2, and Question 4",
                step_builder.find_element_by_css_selector('.correct-list').text
            )

        if max_attempts == 2:
            self.assert_review_conditional_messages_equal(
                step_builder,
                ["Not quite!", "This message is shown when you run out of attempts."]
            )
            self.assert_disabled(controls.try_again)
        else:
            self.assert_review_conditional_messages_equal(step_builder, ["Not quite! You can try again, though."])
            self.assert_clickable(controls.try_again)

        if extended_feedback:
            self.extended_feedback_checks(step_builder, controls, expected_results)

    @data(
        (0, "Yes", "Great!", True, CORRECT),
        (1, "Maybe not", "Ah, damn.", False, INCORRECT),
        (2, "I don't understand", "Really?", False, INCORRECT),
    )
    @unpack
    def test_mcq_feedback(self, choice_index, choice_text, expected_feedback, correct, step_result):
        params = {
            "max_attempts": 1,
            "extended_feedback": True,
        }
        step_builder, controls = self.load_assessment_scenario("step_builder_mcq_feedback.xml", params)

        # Step 1
        # Submit MCQ, go to review step
        self.single_choice_question(None, step_builder, controls, choice_text, step_result, last=True)

        # Check MCQ feedback
        self.review_mcq(step_builder, choice_index, expected_feedback, correct)

        # Reload page
        self.browser.execute_script("$(document).html(' ');")
        step_builder, controls = self.go_to_assessment()

        # Check MCQ feedback
        self.review_mcq(step_builder, choice_index, expected_feedback, correct)

        # Go back to review step
        controls.review_link.click()

        # Check MCQ feedback
        self.review_mcq(step_builder, choice_index, expected_feedback, correct)

    def review_mcq(self, step_builder, choice_index, expected_feedback, correct):
        correctness = 'correct' if correct else 'incorrect'
        mcq_link = step_builder.find_elements_by_css_selector('.{}-list li a'.format(correctness))[0]
        mcq_link.click()
        mcq = step_builder.find_element_by_css_selector(".xblock-v1[data-name='mcq_1_1']")
        self.assert_choice_feedback(mcq, choice_index, expected_feedback, correct)

    def assert_choice_feedback(self, mcq, choice_index, expected_text, correct=True):
        """
        Asserts that feedback for given element contains particular text
        """
        choice = mcq.find_elements_by_css_selector(".choices-list .choice")[choice_index]
        choice_result = choice.find_element_by_css_selector('.choice-result')
        feedback_popup = choice.find_element_by_css_selector(".choice-tips")

        self.wait_until_visible(feedback_popup)
        self.assertEqual(feedback_popup.text, expected_text)
        self.assert_choice_result(choice_result, correct)

    def assert_choice_result(self, choice_result, correct):
        if correct:
            checkmark_class = 'checkmark-correct'
            checkmark_label = 'Correct'
        else:
            checkmark_class = 'checkmark-incorrect'
            checkmark_label = 'Incorrect'

        result_classes = choice_result.get_attribute('class').split()
        result_label = choice_result.get_attribute('aria-label').strip()
        self.wait_until_visible(choice_result)
        self.assertIn(checkmark_class, result_classes)
        self.assertEquals(checkmark_label, result_label)

    def test_review_tips(self):
        params = {
            "max_attempts": 3,
            "extended_feedback": False,
            "include_review_tips": True
        }
        step_builder, controls = self.load_assessment_scenario("step_builder.xml", params)

        # Get one question wrong and one partially wrong on attempt 1 of 3: ####################
        self.freeform_answer(None, step_builder, controls, 'This is the answer', CORRECT)
        self.single_choice_question(None, step_builder, controls, 'Maybe not', INCORRECT)
        self.rating_question(None, step_builder, controls, "5 - Extremely good", CORRECT)
        self.html_section(step_builder, controls)
        self.multiple_response_question(None, step_builder, controls, ("Its beauty",), PARTIAL, last=True)

        # The review tips for MCQ 2 and the MRQ should be shown:
        review_tips_intro = step_builder.find_element_by_css_selector('.review-tips-intro')
        self.assertEqual(
            review_tips_intro.text,
            "You might consider reviewing the following items before your next assessment attempt:"
        )
        review_tips = step_builder.find_element_by_css_selector('.review-tips-list')
        self.assertTrue(review_tips.is_displayed())
        self.assertIn('Take another look at Lesson 1', review_tips.text)
        self.assertNotIn('Lesson 2', review_tips.text)  # This MCQ was correct
        self.assertIn('Take another look at Lesson 3', review_tips.text)
        # If attempts remain and student got some answers wrong, show "incomplete" message
        self.assert_review_conditional_messages_equal(
            step_builder,
            ["Not quite! You can try again, though."]
        )

        # Try again
        self.assert_clickable(controls.try_again)
        controls.try_again.click()

        # Get no questions wrong on attempt 2 of 3: ############################################
        self.freeform_answer(
            None, step_builder, controls, 'This is the answer', CORRECT, saved_value='This is the answer'
        )
        self.single_choice_question(None, step_builder, controls, 'Yes', CORRECT)
        self.rating_question(None, step_builder, controls, "5 - Extremely good", CORRECT)
        user_selection = ("Its elegance", "Its beauty", "Its gracefulness")
        self.html_section(step_builder, controls)
        self.multiple_response_question(None, step_builder, controls, user_selection, CORRECT, last=True)

        # We expect to see this congratulatory message now:
        self.assert_review_conditional_messages_equal(step_builder, ["Great job!"])
        # And no review tips should be shown:
        self.assertEqual(len(step_builder.find_elements_by_css_selector('.review-tips-intro')), 0)

        # Try again
        self.assert_clickable(controls.try_again)
        controls.try_again.click()

        # Get some questions wrong again on attempt 3 of 3:
        self.freeform_answer(
            None, step_builder, controls, 'This is the answer', CORRECT, saved_value='This is the answer'
        )
        self.single_choice_question(None, step_builder, controls, 'Maybe not', INCORRECT)
        self.rating_question(None, step_builder, controls, "1 - Not good at all", INCORRECT)
        self.html_section(step_builder, controls)
        self.multiple_response_question(None, step_builder, controls, ("Its beauty",), PARTIAL, last=True)

        # The review tips will not be shown because no attempts remain:
        self.assertEqual(len(step_builder.find_elements_by_css_selector('.review-tips-intro')), 0)

    @data(True, False)
    def test_conditional_messages(self, include_messages):
        # Test that conditional messages in the review step are visible or not, as appropriate.
        max_attempts = 3
        extended_feedback = False
        params = {
            "max_attempts": max_attempts,
            "extended_feedback": extended_feedback,
            "include_messages": include_messages,
        }
        step_builder, controls = self.load_assessment_scenario("step_builder_conditional_messages.xml", params)

        # First attempt: incomplete (second question wrong)

        # Step 1
        # Submit free-form answer, go to next step
        self.freeform_answer(None, step_builder, controls, 'This is the answer', CORRECT)

        # Step 2
        # Submit MCQ, go to next step
        self.single_choice_question(None, step_builder, controls, 'Maybe not', INCORRECT, last=True)

        # Review step
        expected_results = {
            "correct": 1, "partial": 0, "incorrect": 1, "percentage": 50,
            "num_attempts": 1, "max_attempts": max_attempts
        }
        self.peek_at_review(step_builder, controls, expected_results, extended_feedback=extended_feedback)

        # Should show the following message for incomplete submission
        self.assert_review_conditional_messages_equal(
            step_builder,
            ["Not quite! You can try again, though."] if include_messages else []
        )

        # Try again
        controls.try_again.click()

        self.wait_until_hidden(controls.try_again)

        # Second attempt: complete (both questions correct)

        # Step 1
        # Submit free-form answer, go to next step
        self.freeform_answer(
            None, step_builder, controls, 'This is a different answer', CORRECT, saved_value='This is the answer'
        )
        # Step 2
        # Submit MCQ, go to next step
        self.single_choice_question(None, step_builder, controls, 'Yes', CORRECT, last=True)

        # Review step
        expected_results = {
            "correct": 2, "partial": 0, "incorrect": 0, "percentage": 100,
            "num_attempts": 2, "max_attempts": max_attempts
        }
        self.peek_at_review(step_builder, controls, expected_results, extended_feedback=extended_feedback)

        # Should show the following message for perfect ("complete") submission
        self.assert_review_conditional_messages_equal(
            step_builder,
            ["Great job!"] if include_messages else []
        )

        # Try again
        controls.try_again.click()

        self.wait_until_hidden(controls.try_again)

        # Last attempt: complete (both questions correct)

        # Step 1
        # Submit free-form answer, go to next step
        self.freeform_answer(
            None, step_builder, controls, 'This is yet another answer', CORRECT,
            saved_value='This is a different answer'
        )
        # Step 2
        # Submit MCQ, go to next step
        self.single_choice_question(None, step_builder, controls, 'Yes', CORRECT, last=True)

        # Review step
        expected_results = {
            "correct": 2, "partial": 0, "incorrect": 0, "percentage": 100,
            "num_attempts": 3, "max_attempts": max_attempts
        }
        self.peek_at_review(step_builder, controls, expected_results, extended_feedback=extended_feedback)

        # Should show the following messages:
        self.assert_review_conditional_messages_equal(
            step_builder,
            ["Great job!", "Note: you have used all attempts. Continue to the next unit."] if include_messages else []
        )

    def answer_rating_question(self, step_number, question_number, step_builder, question, choice_name):
        question_text = self.question_text(question_number)
        self.wait_until_text_in(question_text, step_builder)
        self.assertIn(question, step_builder.text)
        choices = GetChoices(
            step_builder, 'div[data-name="rating_{}_{}"] > .rating'.format(step_number, question_number)
        )
        choices.select(choice_name)

    def submit_and_go_to_next_step(self, controls, last=False, no_questions=False):
        controls.submit.click()
        self.wait_until_clickable(controls.next_question)
        controls.next_question.click()
        if last:
            self.wait_until_hidden(controls.next_question)
        else:
            if no_questions:
                self.wait_until_hidden(controls.submit)
            else:
                self.wait_until_disabled(controls.next_question)

    def plot_controls(self, step_builder):
        class Namespace(object):
            pass

        plot_controls = Namespace()

        plot_controls.default_button = step_builder.find_element_by_css_selector(".plot-default")
        plot_controls.average_button = step_builder.find_element_by_css_selector(".plot-average")
        plot_controls.quadrants_button = step_builder.find_element_by_css_selector(".plot-quadrants")

        return plot_controls

    def additional_plot_controls(self, step_builder):
        class Namespace(object):
            pass

        additional_plot_controls = Namespace()

        additional_plot_controls.teacher_button = step_builder.find_element_by_css_selector(
            "input.plot-overlay.plot-overlay-0"
        )
        additional_plot_controls.researchers_button = step_builder.find_element_by_css_selector(
            "input.plot-overlay.plot-overlay-1"
        )
        additional_plot_controls.sheldon_button = step_builder.find_element_by_css_selector(
            "input.plot-overlay.plot-overlay-2"
        )
        additional_plot_controls.yoda_button = step_builder.find_element_by_css_selector(
            "input.plot-overlay.plot-overlay-3"
        )

        return additional_plot_controls

    def plot_empty(self, step_builder):
        points = step_builder.find_elements_by_css_selector("circle")
        self.assertEquals(points, [])

    def check_quadrant_labels(self, step_builder, plot_controls, hidden, labels=['Q1', 'Q2', 'Q3', 'Q4']):
        quadrant_labels = step_builder.find_elements_by_css_selector(".quadrant-label")
        quadrants_button_border_colors = [
            plot_controls.quadrants_button.value_of_css_property('border-top-color'),
            plot_controls.quadrants_button.value_of_css_property('border-right-color'),
            plot_controls.quadrants_button.value_of_css_property('border-bottom-color'),
            plot_controls.quadrants_button.value_of_css_property('border-left-color'),
        ]
        if hidden:
            self.assertEquals(quadrant_labels, [])
            self.assertTrue(all(bc == HTMLColors.RED for bc in quadrants_button_border_colors))
        else:
            self.assertEquals(len(quadrant_labels), 4)
            self.assertEquals(set(label.text for label in quadrant_labels), set(labels))
            self.assertTrue(all(bc == HTMLColors.GREEN for bc in quadrants_button_border_colors))

    def click_overlay_button(self, overlay_button, overlay_on, color_on=None, color_off=HTMLColors.GREY):
        overlay_button.click()
        time.sleep(3)  # give some time for changes
        button_border_colors = [
            overlay_button.value_of_css_property('border-top-color'),
            overlay_button.value_of_css_property('border-right-color'),
            overlay_button.value_of_css_property('border-bottom-color'),
            overlay_button.value_of_css_property('border-left-color'),
        ]
        if overlay_on:
            for bc in button_border_colors:
                self.assertEqual(bc, color_on)
        else:
            for bc in button_border_colors:
                self.assertEqual(bc, color_off)

    def click_default_button(self, plot_controls, overlay_on, color_on=HTMLColors.GREEN):
        self.click_overlay_button(plot_controls.default_button, overlay_on, color_on)

    def click_average_button(self, plot_controls, overlay_on, color_on=HTMLColors.BLUE):
        self.click_overlay_button(plot_controls.average_button, overlay_on, color_on)

    def check_button_label(self, button, expected_value):
        self.assertEquals(button.get_attribute('value'), expected_value)

    def test_empty_plot(self):
        step_builder, controls = self.load_assessment_scenario("step_builder_plot_defaults.xml", {})

        # Step 1: Questions
        # Provide first rating
        self.answer_rating_question(1, 1, step_builder, "How much do you agree?", "1 - Disagree")
        # Provide second rating
        self.answer_rating_question(1, 2, step_builder, "How important do you think this is?", "5 - Very important")
        # Advance
        self.submit_and_go_to_next_step(controls, last=True)

        # Step 2: Plot
        # Check if plot is empty initially (default overlay on, average overlay off)
        self.plot_empty(step_builder)
        # Obtain references to plot controls
        plot_controls = self.plot_controls(step_builder)
        # Check button labels
        self.check_button_label(plot_controls.default_button, "yours")
        self.check_button_label(plot_controls.average_button, "Average")
        # Check if plot is empty (default overlay off, average overlay off)
        self.click_default_button(plot_controls, overlay_on=False)
        self.plot_empty(step_builder)
        # Check if plot is empty (default overlay off, average overlay on)
        self.click_average_button(plot_controls, overlay_on=True)
        self.plot_empty(step_builder)
        # Check if plot is empty (default overlay on, average overlay on)
        self.click_default_button(plot_controls, overlay_on=True)
        self.plot_empty(step_builder)
        # Check if plot is empty (default overlay on, average overlay off)
        self.click_average_button(plot_controls, overlay_on=False)
        self.plot_empty(step_builder)
        # Check quadrant labels
        self.check_quadrant_labels(step_builder, plot_controls, hidden=True)
        plot_controls.quadrants_button.click()
        self.check_quadrant_labels(step_builder, plot_controls, hidden=False)

    def wait_for_multiple_elements(self, step_builder, selector, expected_number_of_elements):
        def wait_for_elements(container):
            elements = container.find_elements_by_css_selector(selector)
            return len(elements) == expected_number_of_elements

        wait = WebDriverWait(step_builder, self.timeout)
        wait.until(wait_for_elements)

    def check_overlays(self, step_builder, total_num_points, overlays):
        self.wait_for_multiple_elements(step_builder, "circle", total_num_points)

        for overlay in overlays:
            # Check if correct number of points is present
            selector = 'circle' + overlay['selector']
            points = step_builder.find_elements_by_css_selector(selector)
            self.assertEquals(len(points), overlay['num_points'])
            # Check point colors
            point_colors = [
                point.value_of_css_property('fill') for point in points
            ]
            self.assertTrue(all(pc == overlay['point_color'] for pc in point_colors))
            # Check tooltips for points
            tooltips = set([
                point.get_attribute('data-tooltip') for point in points
            ])
            self.assertEquals(tooltips, set(overlay['tooltips']))
            # Check positions
            point_positions = set([
                (point.get_attribute('cx'), point.get_attribute('cy')) for point in points
            ])
            self.assertEquals(point_positions, set(overlay['positions']))

    def test_plot(self):
        step_builder, controls = self.load_assessment_scenario("step_builder_plot.xml", {})

        # Step 1: Questions
        # Provide first rating
        self.answer_rating_question(1, 1, step_builder, "How much do you agree?", "1 - Disagree")
        # Provide second rating
        self.answer_rating_question(1, 2, step_builder, "How important do you think this is?", "5 - Very important")
        # Advance
        self.submit_and_go_to_next_step(controls)

        # Step 2: Questions
        # Provide first rating
        self.answer_rating_question(2, 1, step_builder, "How much do you agree?", "5 - Agree")
        # Provide second rating
        self.answer_rating_question(2, 2, step_builder, "How important do you think this is?", "1 - Not important")
        # Advance
        self.submit_and_go_to_next_step(controls, last=True)

        # Step 2: Plot
        # Obtain references to plot controls
        plot_controls = self.plot_controls(step_builder)
        # Check button labels
        self.check_button_label(plot_controls.default_button, "Custom plot label")
        self.check_button_label(plot_controls.average_button, "Average")
        # Overlay data
        default_overlay = {
            'selector': '.claim-default',
            'num_points': 2,
            'point_color': HTMLColors.ORANGE,
            'tooltips': ['2 + 2 = 5: 1, 5', 'The answer to everything is 42: 5, 1'],
            'positions': [
                ('4', '380'),  # Values computed according to xScale and yScale (cf. plot.js)
                ('20', '396'),  # Values computed according to xScale and yScale (cf. plot.js)
            ],
        }
        average_overlay = {
            'selector': '.claim-average',
            'num_points': 2,
            'point_color': HTMLColors.PURPLE,
            'tooltips': ['2 + 2 = 5: 1, 5', 'The answer to everything is 42: 5, 1'],
            'positions': [
                ('4', '380'),  # Values computed according to xScale and yScale (cf. plot.js)
                ('20', '396'),  # Values computed according to xScale and yScale (cf. plot.js)
            ],
        }
        # Check if plot shows correct overlay(s) initially (default overlay on, average overlay off)
        self.check_overlays(step_builder, total_num_points=2, overlays=[default_overlay])

        # Check if plot shows correct overlay(s) (default overlay on, average overlay on)
        self.click_average_button(plot_controls, overlay_on=True, color_on=HTMLColors.PURPLE)
        self.check_overlays(step_builder, 4, overlays=[default_overlay, average_overlay])

        # Check if plot shows correct overlay(s) (default overlay off, average overlay on)
        self.click_default_button(plot_controls, overlay_on=False)
        self.check_overlays(step_builder, 2, overlays=[average_overlay])

        # Check if plot shows correct overlay(s) (default overlay off, average overlay off)
        self.click_average_button(plot_controls, overlay_on=False)
        self.plot_empty(step_builder)

        # Check if plot shows correct overlay(s) (default overlay on, average overlay off)
        self.click_default_button(plot_controls, overlay_on=True, color_on=HTMLColors.ORANGE)
        self.check_overlays(step_builder, 2, overlays=[default_overlay])

        # Check quadrant labels
        self.check_quadrant_labels(step_builder, plot_controls, hidden=True)
        plot_controls.quadrants_button.click()
        self.check_quadrant_labels(
            step_builder, plot_controls, hidden=False,
            labels=['Custom Q1 label', 'Custom Q2 label', 'Custom Q3 label', 'Custom Q4 label']
        )

    def answer_scale_question(self, question_number, step_builder, question, value):
        self.assertEqual(self.get_slider_value(question_number), 50)
        question_text = self.question_text(question_number)
        self.wait_until_text_in(question_text, step_builder)
        self.assertIn(question, step_builder.text)
        self.set_slider_value(question_number, value)

    def test_plot_with_scale_questions(self):
        step_builder, controls = self.load_assessment_scenario("step_builder_plot_scale_questions.xml", {})

        # Step 1: Questions
        # Provide first rating
        self.answer_scale_question(1, step_builder, "How much do you agree?", 17)
        # Provide second rating
        self.answer_scale_question(2, step_builder, "How important do you think this is?", 83)
        # Advance
        self.submit_and_go_to_next_step(controls)

        # Step 2: Questions
        # Provide first rating
        self.answer_rating_question(2, 1, step_builder, "How much do you agree?", "5 - Agree")
        # Provide second rating
        self.answer_rating_question(2, 2, step_builder, "How important do you think this is?", "1 - Not important")
        # Advance
        self.submit_and_go_to_next_step(controls, last=True)

        # Step 2: Plot
        # Obtain references to plot controls
        plot_controls = self.plot_controls(step_builder)
        # Overlay data
        default_overlay = {
            'selector': '.claim-default',
            'num_points': 2,
            'point_color': HTMLColors.ORANGE,
            'tooltips': ['2 + 2 = 5: 17, 83', 'The answer to everything is 42: 5, 1'],
            'positions': [
                ('68', '68'),  # Values computed according to xScale and yScale (cf. plot.js)
                ('20', '396'),  # Values computed according to xScale and yScale (cf. plot.js)
            ],
        }
        average_overlay = {
            'selector': '.claim-average',
            'num_points': 2,
            'point_color': HTMLColors.PURPLE,
            'tooltips': ['2 + 2 = 5: 17, 83', 'The answer to everything is 42: 5, 1'],
            'positions': [
                ('68', '68'),  # Values computed according to xScale and yScale (cf. plot.js)
                ('20', '396'),  # Values computed according to xScale and yScale (cf. plot.js)
            ],
        }
        # Check if plot shows correct overlay(s) initially (default overlay on, average overlay off)
        self.check_overlays(step_builder, total_num_points=2, overlays=[default_overlay])

        # Check if plot shows correct overlay(s) (default overlay on, average overlay on)
        self.click_average_button(plot_controls, overlay_on=True, color_on=HTMLColors.PURPLE)
        self.check_overlays(step_builder, 4, overlays=[default_overlay, average_overlay])

        # Check if plot shows correct overlay(s) (default overlay off, average overlay on)
        self.click_default_button(plot_controls, overlay_on=False)
        self.check_overlays(step_builder, 2, overlays=[average_overlay])

        # Check if plot shows correct overlay(s) (default overlay off, average overlay off)
        self.click_average_button(plot_controls, overlay_on=False)
        self.plot_empty(step_builder)

        # Check if plot shows correct overlay(s) (default overlay on, average overlay off)
        self.click_default_button(plot_controls, overlay_on=True, color_on=HTMLColors.ORANGE)
        self.check_overlays(step_builder, 2, overlays=[default_overlay])

        # Check quadrant labels
        self.check_quadrant_labels(step_builder, plot_controls, hidden=True)
        plot_controls.quadrants_button.click()
        self.check_quadrant_labels(
            step_builder, plot_controls, hidden=False,
            labels=['Custom Q1 label', 'Custom Q2 label', 'Custom Q3 label', 'Custom Q4 label']
        )

    def check_display_status(self, element, hidden):
        if hidden:
            display_status = element.value_of_css_property('display')
            self.assertEquals(display_status, 'none')
        else:
            # self.wait_until_visible(element)
            display_status = element.value_of_css_property('display')
            self.assertEquals(display_status, 'block')

    def check_plot_info(self, step_builder, hidden, visible_overlays=[], hidden_overlays=[]):
        # Check if plot info is present and visible
        plot_info = step_builder.find_element_by_css_selector(".plot-info")
        self.check_display_status(plot_info, hidden)

        # Check if info about visible overlays is present and visible
        for overlay in visible_overlays:
            overlay_info = plot_info.find_element_by_css_selector(overlay['selector'])
            self.check_display_status(overlay_info, hidden=False)
            description = overlay['description']
            citation = overlay['citation']
            if description is not None or citation is not None:
                overlay_plot_label = overlay_info.find_element_by_css_selector('.overlay-plot-label')
                self.assertEquals(overlay_plot_label.text, overlay['plot_label'])
                text_color = overlay_plot_label.value_of_css_property('color')
                self.assertEquals(text_color, overlay['plot_label_color'])
                if description is not None:
                    overlay_description = overlay_info.find_element_by_css_selector('.overlay-description')
                    self.assertEquals(overlay_description.text, 'Description: ' + description)
                if citation is not None:
                    overlay_citation = overlay_info.find_element_by_css_selector('.overlay-citation')
                    self.assertEquals(overlay_citation.text, 'Source: ' + citation)

        # Check if info about hidden overlays is hidden
        for overlay in hidden_overlays:
            overlay_info = plot_info.find_element_by_css_selector(overlay['selector'])
            self.check_display_status(overlay_info, hidden=True)

    def test_plot_overlays(self):
        step_builder, controls = self.load_assessment_scenario("step_builder_plot_overlays.xml", {})

        # Step 1: Questions
        # Provide first rating
        self.answer_rating_question(1, 1, step_builder, "How much do you agree?", "1 - Disagree")
        # Provide second rating
        self.answer_rating_question(1, 2, step_builder, "How important do you think this is?", "5 - Very important")
        # Advance
        self.submit_and_go_to_next_step(controls)

        # Step 2: Questions
        # Provide first rating
        self.answer_rating_question(2, 1, step_builder, "How much do you agree?", "5 - Agree")
        # Provide second rating
        self.answer_rating_question(2, 2, step_builder, "How important do you think this is?", "1 - Not important")
        # Advance
        self.submit_and_go_to_next_step(controls, last=True)

        # Step 2: Plot
        # Obtain references to plot controls
        additional_plot_controls = self.additional_plot_controls(step_builder)
        # Check button labels
        self.check_button_label(additional_plot_controls.teacher_button, "Teacher")
        self.check_button_label(additional_plot_controls.researchers_button, "Researchers")
        self.check_button_label(additional_plot_controls.sheldon_button, "Sheldon Cooper")
        self.check_button_label(additional_plot_controls.yoda_button, "Yoda")
        # Overlay data
        default_overlay = {
            'selector': '.claim-default',
            'num_points': 2,
            'point_color': HTMLColors.ORANGE,
            'tooltips': ['2 + 2 = 5: 1, 5', 'The answer to everything is 42: 5, 1'],
            'positions': [
                ('4', '380'),  # Values computed according to xScale and yScale (cf. plot.js)
                ('20', '396'),  # Values computed according to xScale and yScale (cf. plot.js)
            ],
        }
        teacher_overlay = {
            'selector': '.plot-overlay.plot-overlay-0',
            'num_points': 2,
            'point_color': HTMLColors.CORAL,
            'tooltips': ['2 + 2 = 5: 2, 3', 'The answer to everything is 42: 4, 2'],
            'positions': [
                ('8', '388'),  # Values computed according to xScale and yScale (cf. plot.js)
                ('16', '392'),  # Values computed according to xScale and yScale (cf. plot.js)
            ],
            'plot_label': 'Teacher',
            'plot_label_color': HTMLColors.CORAL,
            'description': None,
            'citation': None,
        }
        researchers_overlay = {
            'selector': '.plot-overlay.plot-overlay-1',
            'num_points': 2,
            'point_color': HTMLColors.CORNFLOWERBLUE,
            'tooltips': ['2 + 2 = 5: 4, 4', 'The answer to everything is 42: 1, 5'],
            'positions': [
                ('16', '384'),  # Values computed according to xScale and yScale (cf. plot.js)
                ('4', '380'),  # Values computed according to xScale and yScale (cf. plot.js)
            ],
            'plot_label': 'Researchers',
            'plot_label_color': HTMLColors.CORNFLOWERBLUE,
            'description': 'Responses of leading researchers in the field',
            'citation': None,
        }
        sheldon_overlay = {
            'selector': '.plot-overlay.plot-overlay-2',
            'num_points': 2,
            'point_color': HTMLColors.OLIVE,
            'tooltips': ['2 + 2 = 5: 3, 5', 'The answer to everything is 42: 2, 4'],
            'positions': [
                ('12', '380'),  # Values computed according to xScale and yScale (cf. plot.js)
                ('8', '384'),  # Values computed according to xScale and yScale (cf. plot.js)
            ],
            'plot_label': 'Sheldon Cooper',
            'plot_label_color': HTMLColors.OLIVE,
            'description': None,
            'citation': 'The Big Bang Theory',
        }
        yoda_overlay = {
            'selector': '.plot-overlay.plot-overlay-3',
            'num_points': 2,
            'point_color': HTMLColors.CRIMSON,
            'tooltips': ['2 + 2 = 5: 1, 2', 'The answer to everything is 42: 3, 3'],
            'positions': [
                ('4', '392'),  # Values computed according to xScale and yScale (cf. plot.js)
                ('12', '388'),  # Values computed according to xScale and yScale (cf. plot.js)
            ],
            'plot_label': 'Yoda',
            'plot_label_color': HTMLColors.CRIMSON,
            'description': 'Powerful you have become, the dark side I sense in you.',
            'citation': 'Star Wars',
        }

        # Check if plot shows correct overlay(s) initially (default overlay on, additional overlays off)
        self.check_overlays(
            step_builder, 2, overlays=[default_overlay]
        )
        self.check_plot_info(
            step_builder,
            hidden=True,
            visible_overlays=[],
            hidden_overlays=[teacher_overlay, researchers_overlay, sheldon_overlay, yoda_overlay]
        )

        # Turn on additional overlays one by one.
        # - Check if plot shows correct overlay(s)
        # - Check if block displays correct info about plot

        # "Teacher" on
        self.click_overlay_button(
            additional_plot_controls.teacher_button, overlay_on=True, color_on=HTMLColors.CORAL
        )
        self.check_overlays(
            step_builder, 4, overlays=[default_overlay, teacher_overlay]
        )
        self.check_plot_info(
            step_builder,
            hidden=True,  # "Teacher" overlay has no description/citation,
                          # so plot info as a whole should stay hidden
            visible_overlays=[teacher_overlay],
            hidden_overlays=[researchers_overlay, sheldon_overlay, yoda_overlay]
        )

        # "Researchers" on
        self.click_overlay_button(
            additional_plot_controls.researchers_button, overlay_on=True, color_on=HTMLColors.CORNFLOWERBLUE
        )
        self.check_overlays(
            step_builder, 6, overlays=[default_overlay, teacher_overlay, researchers_overlay]
        )
        self.check_plot_info(
            step_builder,
            hidden=False,
            visible_overlays=[teacher_overlay, researchers_overlay],
            hidden_overlays=[sheldon_overlay, yoda_overlay]
        )

        # "Sheldon Cooper" on
        self.click_overlay_button(
            additional_plot_controls.sheldon_button, overlay_on=True, color_on=HTMLColors.OLIVE
        )
        self.check_overlays(
            step_builder, 8, overlays=[default_overlay, teacher_overlay, researchers_overlay, sheldon_overlay]
        )
        self.check_plot_info(
            step_builder,
            hidden=False,
            visible_overlays=[teacher_overlay, researchers_overlay, sheldon_overlay],
            hidden_overlays=[yoda_overlay]
        )

        # "Yoda" on
        self.click_overlay_button(
            additional_plot_controls.yoda_button, overlay_on=True, color_on=HTMLColors.CRIMSON
        )
        self.check_overlays(
            step_builder,
            10,
            overlays=[default_overlay, teacher_overlay, researchers_overlay, sheldon_overlay, yoda_overlay]
        )
        self.check_plot_info(
            step_builder,
            hidden=False,
            visible_overlays=[teacher_overlay, researchers_overlay, sheldon_overlay, yoda_overlay],
            hidden_overlays=[]
        )

        # Turn off additional overlays one by one.
        # - Check if plot shows correct overlay(s)
        # - Check if block displays correct info about plot

        # "Yoda" off
        self.click_overlay_button(additional_plot_controls.yoda_button, overlay_on=False)
        self.check_overlays(
            step_builder, 8, overlays=[default_overlay, teacher_overlay, researchers_overlay, sheldon_overlay]
        )
        self.check_plot_info(
            step_builder,
            hidden=False,
            visible_overlays=[teacher_overlay, researchers_overlay, sheldon_overlay],
            hidden_overlays=[yoda_overlay]
        )

        # "Sheldon Cooper" off
        self.click_overlay_button(additional_plot_controls.sheldon_button, overlay_on=False)
        self.check_overlays(
            step_builder, 6, overlays=[default_overlay, teacher_overlay, researchers_overlay]
        )
        self.check_plot_info(
            step_builder,
            hidden=False,
            visible_overlays=[teacher_overlay, researchers_overlay],
            hidden_overlays=[sheldon_overlay, yoda_overlay]
        )

        # "Researchers" off
        self.click_overlay_button(additional_plot_controls.researchers_button, overlay_on=False)
        self.check_overlays(
            step_builder, 4, overlays=[default_overlay, teacher_overlay]
        )
        self.check_plot_info(
            step_builder,
            hidden=True,  # "Teacher" overlay has no description/citation,
                          # so plot info should be hidden at this point
            visible_overlays=[teacher_overlay],
            hidden_overlays=[researchers_overlay, sheldon_overlay, yoda_overlay]
        )

        # "Teacher" off
        self.click_overlay_button(additional_plot_controls.teacher_button, overlay_on=False)
        self.check_overlays(
            step_builder, 2, overlays=[default_overlay]
        )
        self.check_plot_info(
            step_builder,
            hidden=True,
            visible_overlays=[],
            hidden_overlays=[teacher_overlay, researchers_overlay, sheldon_overlay, yoda_overlay]
        )

        # When deactivating an overlay that has no description/citation,
        # visibility of information about remaining overlays that are currently active
        # should not be affected:

        # "Yoda" on:
        self.click_overlay_button(
            additional_plot_controls.yoda_button, overlay_on=True, color_on=HTMLColors.CRIMSON
        )
        self.check_overlays(
            step_builder, 4, overlays=[default_overlay, yoda_overlay]
        )
        self.check_plot_info(
            step_builder,
            hidden=False,  # Plot info becomes visible
            visible_overlays=[yoda_overlay],
            hidden_overlays=[teacher_overlay, researchers_overlay, sheldon_overlay]
        )

        # "Teacher" on:
        self.click_overlay_button(
            additional_plot_controls.teacher_button, overlay_on=True, color_on=HTMLColors.CORAL
        )
        self.check_overlays(
            step_builder, 6, overlays=[default_overlay, yoda_overlay, teacher_overlay]
        )
        self.check_plot_info(
            step_builder,
            hidden=False,
            visible_overlays=[yoda_overlay, teacher_overlay],
            hidden_overlays=[researchers_overlay, sheldon_overlay]
        )

        # "Teacher" off:
        self.click_overlay_button(additional_plot_controls.teacher_button, overlay_on=False)
        self.check_overlays(
            step_builder, 4, overlays=[default_overlay, yoda_overlay]
        )
        self.check_plot_info(
            step_builder,
            hidden=False,  # Plot info stays visible
            visible_overlays=[yoda_overlay],
            hidden_overlays=[teacher_overlay, researchers_overlay, sheldon_overlay],
        )

        # "Yoda" off:
        self.click_overlay_button(additional_plot_controls.yoda_button, overlay_on=False)
        self.check_overlays(
            step_builder, 2, overlays=[default_overlay]
        )
        self.check_plot_info(
            step_builder,
            hidden=True,  # Last remaining overlay with description/citation deactivated;
                          # plot info now hidden
            visible_overlays=[],
            hidden_overlays=[teacher_overlay, researchers_overlay, sheldon_overlay, yoda_overlay]
        )

    def test_instruction_message(self):
        step_builder, controls = self.load_assessment_scenario("step_builder_instruction.xml", {})
        # Step 1
        # Submit free-form answer, go to next step
        self.freeform_answer(None, step_builder, controls, 'This is the answer', CORRECT, hold=True, last=True)
        message = step_builder.find_element_by_css_selector('.sb-step-message')
        self.wait_until_visible(message)
        self.assertEqual(message.text, 'Hello!')
        # Clicking in general should dismiss this message.
        self.browser.execute_script("$(document).trigger('click')")
        self.wait_until_hidden(message)

    def test_no_review_step(self):
        step_builder, controls = self.load_assessment_scenario("step_builder_no_review_step.xml", {})
        # If client-side code tries to call a method on reviewStep
        # even if Step Builder block does not contain a review step,
        # the current step will fail to render.
        # In that case, submitting an answer and will fail,
        # as it requires the corresponding question to be visible:
        self.freeform_answer(None, step_builder, controls, 'This is the answer', CORRECT)

    def provide_freeform_answer(self, step_number, question_number, step_builder, text_input):
        steps = step_builder.find_elements_by_css_selector('div[data-block-type="sb-step"]')
        current_step = steps[step_number-1]
        freeform_questions = current_step.find_elements_by_css_selector('div[data-block-type="pb-answer"]')
        current_question = freeform_questions[question_number-1]

        question_text = self.question_text(question_number)
        self.wait_until_text_in(question_text, current_question)
        self.assertIn("What is your goal?", current_question.text)

        textarea = current_question.find_element_by_css_selector("textarea")
        textarea.clear()
        textarea.send_keys(text_input)
        self.assertEquals(textarea.get_attribute("value"), text_input)

    def submit_and_go_to_review_step(self, step_builder, controls, result):
        controls.submit.click()

        self.do_submit_wait(controls, last=True)
        self._assert_checkmark(step_builder, result)

        self.do_post(controls, last=True)

    def check_viewport(self):
        step_builder_offset = int(self.browser.execute_script(
            "return $('div[data-block-type=\"step-builder\"]').offset().top")
        )

        def is_scrolled_to_top(driver):
            scroll_top = int(driver.execute_script("return $(window).scrollTop()"))
            tolerance = 2
            return abs(scroll_top - step_builder_offset) <= tolerance

        wait = WebDriverWait(self.browser, 5)
        wait.until(is_scrolled_to_top)

    def scroll_down(self):
        self.browser.execute_script("$(window).scrollTop(50)")

    def test_scroll_into_view(self):
        # Make window small, so that we have to scroll.
        self.browser.set_window_size(600, 400)
        step_builder, controls = self.load_assessment_scenario("step_builder_long_steps.xml", {})
        # First step
        self.check_viewport()
        # - Answer questions
        self.provide_freeform_answer(1, 1, step_builder, "This is the answer")
        self.provide_freeform_answer(1, 2, step_builder, "This is the answer")
        self.provide_freeform_answer(1, 3, step_builder, "This is the answer")
        self.provide_freeform_answer(1, 4, step_builder, "This is the answer")
        self.provide_freeform_answer(1, 5, step_builder, "This is the answer")
        # - Submit and go to next step
        self.submit_and_go_to_next_step(controls, no_questions=True)
        # Second step
        self.check_viewport()
        self.scroll_down()
        self.html_section(step_builder, controls)
        # Last step
        self.check_viewport()
        # - Answer questions
        self.provide_freeform_answer(3, 1, step_builder, "This is the answer")
        self.provide_freeform_answer(3, 2, step_builder, "This is the answer")
        self.provide_freeform_answer(3, 3, step_builder, "This is the answer")
        self.provide_freeform_answer(3, 4, step_builder, "This is the answer")
        self.provide_freeform_answer(3, 5, step_builder, "This is the answer")
        # - Submit and go to review step
        self.submit_and_go_to_review_step(step_builder, controls, result=CORRECT)
        # Review step
        self.check_viewport()
        question_links = step_builder.find_elements_by_css_selector('.correct-list li a')
        # - Review questions belonging to first step
        question_links[2].click()
        self.check_viewport()
        self.scroll_down()
        # - Jump to review step
        controls.review_link.click()
        self.check_viewport()
        self.scroll_down()
        # - Review questions belonging to last step
        question_links[7].click()
        self.check_viewport()
        self.scroll_down()
        # - Jump to review step
        controls.review_link.click()
        self.check_viewport()
        self.scroll_down()
        # - Review questions belonging to first step
        question_links[2].click()
        self.check_viewport()
        self.scroll_down()
        # - Navigate to second step
        controls.next_question.click()
        self.check_viewport()
        self.scroll_down()
        # - Review questions belonging to last step
        controls.next_question.click()
        self.check_viewport()
        self.scroll_down()
        # - Navigate to review step
        controls.review.click()
        self.check_viewport()



# -*- coding: utf-8 -*-
#
# Copyright (c) 2014-2015 Harvard, edX & OpenCraft
#
# This software's license gives you freedom; you can copy, convey,
# propagate, redistribute and/or modify this program under the terms of
# the GNU Affero General Public License (AGPL) as published by the Free
# Software Foundation (FSF), either version 3 of the License, or (at your
# option) any later version of the AGPL published by the FSF.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero
# General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program in a file in the toplevel directory called
# "AGPLv3".  If not, see <http://www.gnu.org/licenses/>.
#
"""
Test that <span class="pb-clarification"> elements are transformed into LMS-like tooltips.
"""

from cgi import escape

# Imports ###########################################################
import ddt
from xblockutils.base_test import SeleniumXBlockTest

# Classes ###########################################################


@ddt.ddt
class ClarificationTest(SeleniumXBlockTest):
    """
    Test that the content of span.pb-clarification elements is transformed into
    tooltip elements.
    """

    clarification_text = 'Let me clarify...'

    mcq_template = """
        <problem-builder>
            <pb-mcq question="Who was your favorite character? {clarify_escaped}">
                <pb-choice value="gaius">Gaius Baltar</pb-choice>
                <pb-choice value="adama">Admiral William Adama {clarify}</pb-choice>
                <pb-choice value="starbuck">Starbuck</pb-choice>
            </pb-mcq>
        </problem-builder>
    """

    mrq_template = """
        <problem-builder>
            <pb-mrq question="What makes a great {clarify_escaped} MRQ {clarify_escaped}?">
                <pb-choice value="1">Lots of choices</pb-choice>
                <pb-choice value="2">Funny{clarify} choices</pb-choice>
                <pb-choice value="3">Not sure {clarify}</pb-choice>
            </pb-mrq>
        </problem-builder>
    """

    rating_template = """
        <problem-builder>
            <pb-rating name="rating_1_1" question="How do you rate {clarify_escaped} Battlestar Galactica?">
                <pb-choice value="6">More than 5 stars {clarify}</pb-choice>
            </pb-rating>
        </problem-builder>
    """

    long_answer_template = """
        <problem-builder>
            <pb-answer question="What did you think {clarify_escaped} of the ending?" />
        </problem-builder>
    """

    html_block_template = """
        <problem-builder>
            <html_demo><p>This is some raw {clarify} HTML code.</p></html_demo>
        </problem-builder>
    """

    def prepare_xml_scenario(self, xml_template):
        span = '<span class="pb-clarification">{}</span>'.format(self.clarification_text)
        escaped_span = escape(span, quote=True)
        return xml_template.format(
            clarify=span,
            clarify_escaped=escaped_span
        )

    @ddt.data(
        (mcq_template, 2),
        (mrq_template, 4),
        (rating_template, 2),
        (long_answer_template, 1),
        (html_block_template, 1),
    )
    @ddt.unpack
    def test_title(self, xml_template, tooltip_count):
        self.set_scenario_xml(self.prepare_xml_scenario(xml_template))
        pb_element = self.go_to_view()
        clarifications = pb_element.find_elements_by_css_selector('span.pb-clarification')

        self.assertEqual(len(clarifications), tooltip_count)

        for clarification in clarifications:
            tooltip = clarification.find_element_by_css_selector('i[data-tooltip]')
            self.assertEqual(tooltip.get_attribute('data-tooltip'), self.clarification_text)

            sr = clarification.find_element_by_css_selector('span.sr')
            self.assertEqual(sr.text, self.clarification_text)



import time

from django.core.management.base import BaseCommand

from problem_builder.models import Answer


class Command(BaseCommand):
    """
    Copy content of the deprecated Answer.course_id column into Answer.course_key in batches.
    The batch size and sleep time between batches are configurable.
    """
    help = 'Copy content of the deprecated Answer.course_id column into Answer.course_key in batches'

    def add_arguments(self, parser):
        parser.add_argument(
            '--batch-size',
            help='The size of each batch of records to copy (default: 5000).',
            type=int,
            default=5000,
        )
        parser.add_argument(
            '--sleep',
            help='Number of seconds to sleep before processing the next batch (default: 5).',
            type=int,
            default=5
        )

    def handle(self, *args, **options):
        batch_size = options['batch_size']
        sleep_time = options['sleep']
        queryset = Answer.objects.filter(course_key__isnull=True)

        self.stdout.write(
            "Copying Answer.course_id field into Answer.course_key in batches of {}".format(batch_size)
        )

        idx = 0
        while True:
            idx += 1
            batch = queryset[:batch_size]
            if not batch:
                break
            for answer in batch:
                answer.course_key = answer.course_id
                answer.save()
            self.stdout.write("Processed batch {}".format(idx))
            time.sleep(sleep_time)

        self.stdout.write("Successfully copied Answer.course_id into Answer.course_key")



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('problem_builder', '0002_auto_20160121_1525'),
    ]

    operations = [
        migrations.AddField(
            model_name='answer',
            name='course_key',
            field=models.CharField(default=None, max_length=255, null=True, db_index=True),
        ),
        migrations.AlterUniqueTogether(
            name='answer',
            unique_together=set([('student_id', 'course_key', 'name'), ('student_id', 'course_id', 'name')]),
        ),
    ]



# -*- coding: utf-8 -*-
from south.db import db
from south.v2 import SchemaMigration


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'Share'
        db.create_table('problem_builder_share', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('shared_by', self.gf('django.db.models.fields.related.ForeignKey')(
                related_name='problem_builder_shared_by', to=orm['auth.User']
            )),
            ('submission_uid', self.gf('django.db.models.fields.CharField')(max_length=32)),
            ('block_id', self.gf('django.db.models.fields.CharField')(max_length=255, db_index=True)),
            ('shared_with', self.gf('django.db.models.fields.related.ForeignKey')(
                related_name='problem_builder_shared_with', to=orm['auth.User']
            )),
            ('notified', self.gf('django.db.models.fields.BooleanField')(default=False, db_index=True)),
        ))
        db.send_create_signal('problem_builder', ['Share'])

        # Adding unique constraint on 'Share', fields ['shared_by', 'shared_with', 'block_id']
        db.create_unique('problem_builder_share', ['shared_by_id', 'shared_with_id', 'block_id'])

    def backwards(self, orm):
        # Removing unique constraint on 'Share', fields ['shared_by', 'shared_with', 'block_id']
        db.delete_unique('problem_builder_share', ['shared_by_id', 'shared_with_id', 'block_id'])

        # Deleting model 'Share'
        db.delete_table('problem_builder_share')

    models = {
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {
                'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'
            })
        },
        'auth.permission': {
            'Meta': {
                'ordering': "('content_type__app_label', 'content_type__model', 'codename')",
                'unique_together': "(('content_type', 'codename'),)", 'object_name': 'Permission'
            },
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {
                'to': "orm['contenttypes.ContentType']"
            }),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {
                'to': "orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'
            }),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {
                'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'
            }),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'contenttypes.contenttype': {
            'Meta': {
                'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)",
                'object_name': 'ContentType', 'db_table': "'django_content_type'"
            },
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'problem_builder.answer': {
            'Meta': {'unique_together': "(('student_id', 'course_id', 'name'),)", 'object_name': 'Answer'},
            'course_id': ('django.db.models.fields.CharField', [], {'max_length': '50', 'db_index': 'True'}),
            'created_on': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'modified_on': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50', 'db_index': 'True'}),
            'student_id': ('django.db.models.fields.CharField', [], {'max_length': '32', 'db_index': 'True'}),
            'student_input': ('django.db.models.fields.TextField', [], {'default': "''", 'blank': 'True'})
        },
        'problem_builder.share': {
            'Meta': {'unique_together': "(('shared_by', 'shared_with', 'block_id'),)", 'object_name': 'Share'},
            'block_id': ('django.db.models.fields.CharField', [], {'max_length': '255', 'db_index': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'notified': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'db_index': 'True'}),
            'shared_by': ('django.db.models.fields.related.ForeignKey', [], {
                'related_name': "'problem_builder_shared_by'", 'to': "orm['auth.User']"
            }),
            'shared_with': ('django.db.models.fields.related.ForeignKey', [], {
                'related_name': "'problem_builder_shared_with'", 'to': "orm['auth.User']"
            }),
            'submission_uid': ('django.db.models.fields.CharField', [], {'max_length': '32'})
        }
    }

    complete_apps = ['problem_builder']



# -*- coding: utf-8 -*-
#
# Copyright (c) 2014-2015 Harvard, edX & OpenCraft
#
# This software's license gives you freedom; you can copy, convey,
# propagate, redistribute and/or modify this program under the terms of
# the GNU Affero General Public License (AGPL) as published by the Free
# Software Foundation (FSF), either version 3 of the License, or (at your
# option) any later version of the AGPL published by the FSF.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero
# General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program in a file in the toplevel directory called
# "AGPLv3".  If not, see <http://www.gnu.org/licenses/>.
#

# Imports ###########################################################

from mock import patch

from workbench.runtime import WorkbenchRuntime

from .base_test import MentoringBaseTest

# Classes ###########################################################


class MentoringTableBlockTest(MentoringBaseTest):

    def test_mentoring_table(self):
        # Initially, the table should be blank, with just the titles
        table = self.go_to_page('Table 2', css_selector='.mentoring-table')
        headers = table.find_elements_by_css_selector('th')
        self.assertEqual(len(headers), 2)
        self.assertEqual(headers[0].text, 'Header Test 1')
        self.assertEqual(headers[1].text, 'Header Test 2')

        rows = table.find_elements_by_css_selector('td')
        self.assertEqual(len(rows), 2)
        self.assertEqual(rows[0].text, 'No answer yet.')
        self.assertEqual(rows[1].text, 'No answer yet.')

        # Fill the answers - they should appear in the table
        mentoring = self.go_to_page('Table 1')
        answers = mentoring.find_elements_by_css_selector('textarea')
        answers[0].send_keys('This is the answer #1')
        answers[1].send_keys('This is the answer #2')
        submit = mentoring.find_element_by_css_selector('.submit input.input-main')
        submit.click()
        self.wait_until_disabled(submit)

        table = self.go_to_page('Table 2', css_selector='.mentoring-table')
        rows = table.find_elements_by_css_selector('td')
        self.assertEqual(len(rows), 2)
        self.assertEqual(rows[0].text, 'This is the answer #1')
        self.assertEqual(rows[1].text, 'This is the answer #2')

        # Ensure that table block makes an effort to translate URLs in column headers
        link_template = "<a href='http://www.test.com'>{}</a> in a column header."
        original_contents = link_template.format('Link')
        updated_contents = link_template.format('Updated link')

        with patch.object(WorkbenchRuntime, 'replace_jump_to_id_urls', create=True) as patched_method:
            patched_method.return_value = updated_contents

            table = self.go_to_page('Table 3', css_selector='.mentoring-table')
            patched_method.assert_called_once_with(original_contents)
            link = table.find_element_by_css_selector('a')
            self.assertEquals(link.text, 'Updated link')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('problem_builder', '0004_copy_course_ids'),
    ]

    operations = [
        migrations.AlterField(
            model_name='answer',
            name='course_id',
            field=models.CharField(default=None, max_length=50, null=True, db_index=True, blank=True),
        ),
        migrations.AlterField(
            model_name='answer',
            name='course_key',
            field=models.CharField(max_length=255, db_index=True),
        ),
    ]



# -*- coding: utf-8 -*-
#
# Copyright (c) 2014-2015 Harvard, edX & OpenCraft
#
# This software's license gives you freedom; you can copy, convey,
# propagate, redistribute and/or modify this program under the terms of
# the GNU Affero General Public License (AGPL) as published by the Free
# Software Foundation (FSF), either version 3 of the License, or (at your
# option) any later version of the AGPL published by the FSF.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero
# General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program in a file in the toplevel directory called
# "AGPLv3".  If not, see <http://www.gnu.org/licenses/>.
#

# Imports ###########################################################

from .base_test import (GetChoices, MentoringAssessmentBaseTest,
                        ProblemBuilderBaseTest)

# Classes ###########################################################


class CompletionBlockTestMixin(object):
    """
    Mixin for testing completion blocks.
    """

    @property
    def checkmarks(self):
        return self.browser.find_elements_by_css_selector('.submit-result')

    @property
    def completion_checkbox(self):
        return self.browser.find_element_by_css_selector('.pb-completion-value')

    @property
    def completion_checkboxes(self):
        return self.browser.find_elements_by_css_selector('.pb-completion-value')

    def expect_checkmarks_visible(self, first_visible, second_visible):
        first_checkmark, second_checkmark = self.checkmarks
        self.assertEqual(first_checkmark.is_displayed(), first_visible)
        self.assertEqual(second_checkmark.is_displayed(), second_visible)

    def expect_checkbox_checked(self, checked):
        self.assertEqual(bool(self.completion_checkbox.get_attribute('checked')), checked)

    def expect_checkboxes_checked(self, first_checked, second_checked):
        first_checkbox, second_checkbox = self.completion_checkboxes
        self.assertEqual(bool(first_checkbox.get_attribute('checked')), first_checked)
        self.assertEqual(bool(second_checkbox.get_attribute('checked')), second_checked)


class CompletionBlockTest(CompletionBlockTestMixin, ProblemBuilderBaseTest):
    """
    Tests for CompletionBlock inside a normal Problem Builder block.
    """

    def test_simple_flow(self):
        """
        Test a regular Problem Builder block containing one completion block.
        """
        self.pb_wrapper = self.load_scenario('completion_problem.xml', {'include_mcq': False})
        self.wait_for_init()

        # Checkbox of completion block should not have "checked" attribute set initially,
        # and "Submit" should be enabled since leaving checkbox unchecked produces a valid value:
        self.assertIsNone(self.completion_checkbox.get_attribute('checked'))
        self.expect_checkmark_visible(False)
        self.expect_submit_enabled(True)

        # Confirm completion by checking checkbox, and click "Submit":
        self.completion_checkbox.click()
        self.click_submit(self.pb_wrapper)
        # Now, we expect "Submit" to be disabled and the checkmark to be visible:
        self.expect_checkbox_checked(True)
        self.expect_checkmark_visible(True)
        self.expect_submit_enabled(False)

        # Uncheck checkbox
        self.completion_checkbox.click()
        # It should be possible to click "Submit" again, and the checkmark should be hidden:
        self.expect_checkbox_checked(False)
        self.expect_checkmark_visible(False)
        self.expect_submit_enabled(True)

        # Now reload the page:
        self.pb_wrapper = self.reload_page()
        self.wait_for_init()
        # The checkbox should be checked (since that's the value we submitted earlier),
        # and "Submit" should be disabled (to discourage submitting the same answer):
        self.expect_checkbox_checked(True)
        self.expect_checkmark_visible(True)
        self.expect_submit_enabled(False)

    def test_simple_flow_with_peer(self):
        """
        Test a regular Problem Builder block containing one completion block and an MCQ.
        """
        self.pb_wrapper = self.load_scenario("completion_problem.xml", {"include_mcq": True})
        self.wait_for_init()

        # Checkbox of completion block should not have "checked" attribute set initially,
        # and "Submit" should be disabled until an MCQ choice is selected
        self.assertIsNone(self.completion_checkbox.get_attribute('checked'))
        self.expect_checkmark_visible(False)
        self.expect_submit_enabled(False)

        # Confirm completion by checking checkbox:
        self.completion_checkbox.click()
        # Checkmark should be hidden, and "Submit" should be disabled (did not select an MCQ choice yet):
        self.expect_checkbox_checked(True)
        self.expect_checkmark_visible(False)
        self.expect_submit_enabled(False)

        # Select an MCQ choice:
        GetChoices(self.pb_wrapper).select('Yes')
        # "Submit" button should now be enabled:
        self.expect_checkmark_visible(False)
        self.expect_submit_enabled(True)

        # Submit answers
        self.click_submit(self.pb_wrapper)
        # Now, we expect submit to be disabled and the checkmark to be visible:
        self.expect_checkmark_visible(True)
        self.expect_submit_enabled(False)

        # Uncheck checkbox
        self.completion_checkbox.click()
        # It should be possible to click "Submit" again, and the checkmark should be hidden:
        self.expect_checkbox_checked(False)
        self.expect_checkmark_visible(False)
        self.expect_submit_enabled(True)

    def test_multiple_completion_blocks(self):
        """
        Test a regular Problem Builder block containing multiple completion blocks.
        """
        self.pb_wrapper = self.load_scenario("completion_multiple_problem.xml")
        self.wait_for_init()

        first_checkbox, second_checkbox = self.completion_checkboxes

        # Checkboxes of completion blocks should not have "checked" attribute set initially,
        # and "Submit" should be enabled since leaving checkboxes unchecked produces a valid value:
        self.assertIsNone(first_checkbox.get_attribute('checked'))
        self.assertIsNone(second_checkbox.get_attribute('checked'))
        self.expect_checkmarks_visible(False, False)
        self.expect_submit_enabled(True)

        # Confirm completion by checking first checkbox:
        first_checkbox.click()

        self.expect_checkboxes_checked(True, False)
        self.expect_checkmarks_visible(False, False)
        self.expect_submit_enabled(True)

        # Submit answers
        self.click_submit(self.pb_wrapper)
        # Now, we expect "Submit" to be disabled and the checkmarks to be visible:
        self.expect_checkboxes_checked(True, False)
        self.expect_checkmarks_visible(True, True)
        self.expect_submit_enabled(False)

        # Uncheck first checkbox
        first_checkbox.click()
        # It should be possible to click "Submit" again, and the checkmarks should be hidden:
        self.expect_checkboxes_checked(False, False)
        self.expect_checkmarks_visible(False, False)
        self.expect_submit_enabled(True)

        # Now reload the page:
        self.pb_wrapper = self.reload_page()
        self.wait_for_init()

        # The first checkbox should be checked, and the second checkbox should be unchecked
        # (since these are the values we submitted earlier);
        # "Submit" should be disabled (to discourage submitting the same answer):
        self.expect_checkboxes_checked(True, False)
        self.expect_checkmarks_visible(True, True)
        self.expect_submit_enabled(False)


class CompletionStepBlockTest(CompletionBlockTestMixin, MentoringAssessmentBaseTest):
    """
    Tests for CompletionBlock inside a Step Builder block.
    """

    def test_step_with_completion_block(self):
        """
        Test a regular Step Builder block containing one completion block and an MCQ.
        """
        step_builder, controls = self.load_assessment_scenario("completion_step.xml")
        self.wait_for_init()
        self.assertIsNone(self.completion_checkbox.get_attribute('checked'))

        # Submit step 1 (the completion block step), and advance to next step:
        question = self.expect_question_visible(1, step_builder, question_text="Attendance Check")
        self.assertIn("Did you attend the meeting?", question.text)  # Question
        self.assertIn("Yes, I did.", question.text)  # Answer
        self.assertTrue(controls.submit.is_enabled())
        self.assert_hidden(controls.try_again)

        self.completion_checkbox.click()
        self.expect_checkbox_checked(True)
        controls.submit.click()
        self.do_submit_wait(controls, last=False)
        self.wait_until_clickable(controls.next_question)
        controls.next_question.click()

        # Submit step 2 (the MCQ step), and advance to review step:
        question = self.expect_question_visible(2, step_builder)
        GetChoices(question).select("Yes")
        controls.submit.click()
        self.do_submit_wait(controls, last=True)
        self.wait_until_clickable(controls.review)
        controls.review.click()

        self.wait_until_visible(controls.try_again)
        # You can't get a completion block question wrong, but it does count as one correct point by default:
        self.assertIn("You answered 2 questions correctly", step_builder.text)



# -*- coding: utf-8 -*-
#
# Copyright (c) 2014-2015 Harvard, edX & OpenCraft
#
# This software's license gives you freedom; you can copy, convey,
# propagate, redistribute and/or modify this program under the terms of
# the GNU Affero General Public License (AGPL) as published by the Free
# Software Foundation (FSF), either version 3 of the License, or (at your
# option) any later version of the AGPL published by the FSF.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero
# General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program in a file in the toplevel directory called
# "AGPLv3".  If not, see <http://www.gnu.org/licenses/>.
#
import mock
from xblock.fields import String
from xblockutils.base_test import SeleniumBaseTest, SeleniumXBlockTest
from xblockutils.resources import ResourceLoader

# Studio adds a url_name property to each XBlock but Workbench doesn't.
# Since we rely on it, we need to mock url_name support so it can be set via XML and
# accessed like a normal field.
from problem_builder.mentoring import MentoringBlock

MentoringBlock.url_name = String()

loader = ResourceLoader(__name__)

CORRECT, INCORRECT, PARTIAL = "correct", "incorrect", "partially-correct"


class PopupCheckMixin(object):
    """
    Code used by MentoringBaseTest and MentoringAssessmentBaseTest
    """
    def popup_check(self, mentoring, item_feedbacks, prefix='', do_submit=True):

        submit = mentoring.find_element_by_css_selector('.submit input.input-main')

        for index, expected_feedback in enumerate(item_feedbacks):

            self.wait_until_exists(prefix + " .choice")
            choice_wrapper = mentoring.find_elements_by_css_selector(prefix + " .choice")[index]
            if do_submit:
                # clicking on actual radio button
                self.wait_until_exists(".choice-selector input")
                choice_wrapper.find_element_by_css_selector(".choice-selector input").click()
                submit.click()
            self.wait_until_disabled(submit)
            self.wait_until_exists(".choice-result")
            item_feedback_icon = choice_wrapper.find_element_by_css_selector(".choice-result")
            self.wait_until_clickable(item_feedback_icon)
            item_feedback_icon.click()  # clicking on item feedback icon
            self.wait_until_exists(".choice-tips")
            item_feedback_popup = choice_wrapper.find_element_by_css_selector(".choice-tips")
            self.assertTrue(item_feedback_popup.is_displayed())
            self.assertEqual(item_feedback_popup.text, expected_feedback)

            self.wait_until_clickable(item_feedback_popup)
            item_feedback_popup.click()
            self.wait_until_visible(item_feedback_popup)
            self.assertTrue(item_feedback_popup.is_displayed())

            mentoring.find_element_by_css_selector('.title').click()
            self.wait_until_hidden(item_feedback_popup)
            self.assertFalse(item_feedback_popup.is_displayed())


class ProblemBuilderBaseTest(SeleniumXBlockTest, PopupCheckMixin):
    """
    The new base class for integration tests.
    Scenarios can be loaded and edited on the fly.
    """
    default_css_selector = 'div.mentoring'

    def load_scenario(self, xml_file, params=None, load_immediately=True):
        """
        Given the name of an XML file in the xml_templates folder, load it into the workbench.
        """
        params = params or {}
        scenario = loader.render_template("xml_templates/{}".format(xml_file), params)
        self.set_scenario_xml(scenario)
        if load_immediately:
            return self.go_to_view("student_view")

    def go_to_view(self, view_name):
        """ Eliminate errors that come from the Workbench banner overlapping elements """
        element = super(ProblemBuilderBaseTest, self).go_to_view(view_name)
        self.browser.execute_script('document.querySelectorAll("header.banner")[0].style.display="none";')
        return element

    def wait_for_init(self):
        """ Wait for the scenario to initialize """
        self.wait_until_hidden(self.browser.find_element_by_css_selector('.messages'))

    def reload_page(self):
        """
        Reload current page.
        """
        self.browser.execute_script("$(document).html(' ');")
        return self.go_to_view("student_view")

    @property
    def checkmark(self):
        return self.browser.find_element_by_css_selector('.submit-result')

    @property
    def submit_button(self):
        return self.browser.find_element_by_css_selector('.submit input.input-main')

    def click_submit(self, mentoring):
        """ Click the submit button and wait for the response """
        submit = mentoring.find_element_by_css_selector('.submit input.input-main')
        self.assertTrue(submit.is_displayed())
        self.assertTrue(submit.is_enabled())
        submit.click()
        self.wait_until_disabled(submit)

    def click_choice(self, container, choice_text):
        """ Click on the choice label with the specified text """
        for label in container.find_elements_by_css_selector('.choice label'):
            if choice_text in label.text:
                label.click()
                break

    def expect_checkmark_visible(self, visible):
        if visible:
            self.wait_until_visible(self.checkmark)
        else:
            self.wait_until_hidden(self.checkmark)
        self.assertEqual(self.checkmark.is_displayed(), visible)

    def expect_submit_enabled(self, enabled):
        if enabled:
            self.wait_until_clickable(self.submit_button)
        else:
            self.wait_until_disabled(self.submit_button)
        self.assertEqual(self.submit_button.is_enabled(), enabled)


class MentoringBaseTest(SeleniumBaseTest, PopupCheckMixin):
    module_name = __name__
    default_css_selector = 'div.mentoring'

    __asides_patch = None

    def go_to_page(self, page_title, **kwargs):
        """ Eliminate errors that come from the Workbench banner overlapping elements """
        element = super(MentoringBaseTest, self).go_to_page(page_title, **kwargs)
        self.browser.execute_script('document.querySelectorAll("header.banner")[0].style.display="none";')
        return element

    @classmethod
    def setUpClass(cls):
        super(MentoringBaseTest, cls).setUpClass()
        cls.__asides_patch = mock.patch(
            "workbench.runtime.WorkbenchRuntime.applicable_aside_types",
            mock.Mock(return_value=[])
        )
        cls.__asides_patch.start()

    @classmethod
    def tearDownClass(cls):
        cls.__asides_patch.stop()
        super(MentoringBaseTest, cls).tearDownClass()


class MentoringAssessmentBaseTest(ProblemBuilderBaseTest):
    """
    Base class for tests of assessment mode
    """
    @staticmethod
    def question_text(number):
        if number:
            return "Question %s" % number
        else:
            return "Question"

    def load_assessment_scenario(self, xml_file, params=None):
        """ Loads an assessment scenario from an XML template """
        params = params or {}
        scenario = loader.render_template("xml_templates/{}".format(xml_file), params)
        self.set_scenario_xml(scenario)
        return self.go_to_assessment()

    def go_to_assessment(self):
        """ Navigates to assessment page """
        mentoring = self.go_to_view("student_view")

        class Namespace(object):
            pass

        controls = Namespace()

        controls.submit = mentoring.find_element_by_css_selector("input.input-main")
        controls.next_question = mentoring.find_element_by_css_selector("input.input-next")
        controls.review = mentoring.find_element_by_css_selector("input.input-review")
        controls.try_again = mentoring.find_element_by_css_selector("input.input-try-again")
        controls.review_link = mentoring.find_element_by_css_selector(".review-link a")

        return mentoring, controls

    def wait_for_init(self):
        """ Wait for the scenario to initialize """
        self.wait_until_visible(self.browser.find_elements_by_css_selector('.sb-step')[0])

    def assert_hidden(self, elem):
        self.assertFalse(elem.is_displayed())

    def assert_disabled(self, elem):
        self.assertTrue(elem.is_displayed())
        self.assertFalse(elem.is_enabled())

    def assert_clickable(self, elem):
        self.assertTrue(elem.is_displayed())
        self.assertTrue(elem.is_enabled())

    def ending_controls(self, controls, last):
        if last:
            self.assert_hidden(controls.next_question)
            self.assert_disabled(controls.review)
        else:
            self.assert_disabled(controls.next_question)
            self.assert_hidden(controls.review)

    def selected_controls(self, controls, last):
        self.assert_clickable(controls.submit)
        self.ending_controls(controls, last)

    def assert_message_text(self, mentoring, text):
        message_wrapper = mentoring.find_element_by_css_selector('.assessment-message')
        self.assertEqual(message_wrapper.text, text)
        self.assertTrue(message_wrapper.is_displayed())

    def assert_no_message_text(self, mentoring):
        message_wrapper = mentoring.find_element_by_css_selector('.assessment-message')
        self.assertEqual(message_wrapper.text, '')

    def check_question_feedback(self, step_builder, question):
        question_checkmark = step_builder.find_element_by_css_selector('.assessment-checkmark')
        question_feedback = question.find_element_by_css_selector(".feedback")
        self.assertTrue(question_feedback.is_displayed())
        self.assertEqual(question_feedback.text, "Question Feedback Message")

        question.click()
        self.assertFalse(question_feedback.is_displayed())

        question_checkmark.click()
        self.assertTrue(question_feedback.is_displayed())

    def do_submit_wait(self, controls, last):
        if last:
            self.wait_until_clickable(controls.review)
        else:
            self.wait_until_clickable(controls.next_question)

    def do_post(self, controls, last):
        if last:
            controls.review.click()
        else:
            controls.next_question.click()

    def multiple_response_question(self, number, mentoring, controls, choice_names, result, last=False):
        question = self.peek_at_multiple_response_question(number, mentoring, controls, last=last)

        choices = GetChoices(question)
        expected_choices = {
            "Its elegance": False,
            "Its beauty": False,
            "Its gracefulness": False,
            "Its bugs": False,
        }
        self.assertEquals(choices.state, expected_choices)

        for name in choice_names:
            choices.select(name)
            expected_choices[name] = True

        self.assertEquals(choices.state, expected_choices)

        self.selected_controls(controls, last)

        controls.submit.click()

        self.do_submit_wait(controls, last)
        self._assert_checkmark(mentoring, result)
        controls.review.click()

    def expect_question_visible(self, number, mentoring, question_text=None):
        if not question_text:
            question_text = self.question_text(number)
        self.wait_until_text_in(question_text, mentoring)
        question_div = None
        for xblock_div in mentoring.find_elements_by_css_selector('div.xblock-v1'):
            header_text = xblock_div.find_elements_by_css_selector('h4.question-title')
            if header_text and question_text in header_text[0].text:
                question_div = xblock_div
                self.assertTrue(xblock_div.is_displayed())
            elif header_text:
                self.assertFalse(xblock_div.is_displayed())
            # else this is an HTML block or something else, not a question step

        self.assertIsNotNone(question_div)
        return question_div

    def answer_mcq(self, number, name, value, mentoring, controls, is_last=False):
        self.expect_question_visible(number, mentoring)

        mentoring.find_element_by_css_selector('input[name={}][value={}]'.format(name, value)).click()
        controls.submit.click()
        if is_last:
            self.wait_until_clickable(controls.review)
            controls.review.click()
            self.wait_until_hidden(controls.review)
        else:
            self.wait_until_clickable(controls.next_question)
            controls.next_question.click()

    def _assert_checkmark(self, mentoring, result):
        """Assert that only the desired checkmark is present."""
        states = {CORRECT: 0, INCORRECT: 0, PARTIAL: 0}
        states[result] += 1

        for name, count in states.items():
            self.assertEqual(len(mentoring.find_elements_by_css_selector(".submit .checkmark-{}".format(name))), count)


class GetChoices(object):
    """ Helper class for interacting with MCQ options """
    def __init__(self, question, selector=".choices"):
        self._mcq = question.find_element_by_css_selector(selector)

    @property
    def text(self):
        return self._mcq.text

    @property
    def state(self):
        return {
            choice.text: choice.find_element_by_css_selector("input").is_selected()
            for choice in self._mcq.find_elements_by_css_selector(".choice")}

    def select(self, text):
        choice_wrapper = self.get_option_element(text)
        choice_wrapper.find_element_by_css_selector("input").click()

    def get_option_element(self, text):
        for choice in self._mcq.find_elements_by_css_selector(".choice"):
            if choice.text == text:
                return choice
        raise AssertionError("Expected selectable item present: {}".format(text))



# -*- coding: utf-8 -*-
#
# Copyright (c) 2014-2015 Harvard, edX & OpenCraft
#
# This software's license gives you freedom; you can copy, convey,
# propagate, redistribute and/or modify this program under the terms of
# the GNU Affero General Public License (AGPL) as published by the Free
# Software Foundation (FSF), either version 3 of the License, or (at your
# option) any later version of the AGPL published by the FSF.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero
# General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program in a file in the toplevel directory called
# "AGPLv3".  If not, see <http://www.gnu.org/licenses/>.
#
"""
Integrations between these XBlocks and the edX Submissions API
"""

import six
from xblock.completable import XBlockCompletionMode

try:
    from submissions import api as sub_api
except ImportError:
    sub_api = None  # We are probably in the workbench. Don't use the submissions API


class SubmittingXBlockMixin(object):
    """
    Simplifies use of the submissions API by an XBlock.
    """

    completion_mode = XBlockCompletionMode.COMPLETABLE
    has_score = True

    @property
    def student_item_key(self):
        """
        Get the student_item_dict required for the submissions API.
        """
        assert sub_api is not None
        location = self.location.replace(branch=None, version=None)  # Standardize the key in case it isn't already
        return dict(
            student_id=self.runtime.anonymous_student_id,
            course_id=six.text_type(location.course_key),
            item_id=six.text_type(location),
            item_type=self.scope_ids.block_type,
        )



# -*- coding: utf-8 -*-
#
# Copyright (c) 2014-2015 Harvard, edX & OpenCraft
#
# This software's license gives you freedom; you can copy, convey,
# propagate, redistribute and/or modify this program under the terms of
# the GNU Affero General Public License (AGPL) as published by the Free
# Software Foundation (FSF), either version 3 of the License, or (at your
# option) any later version of the AGPL published by the FSF.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero
# General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program in a file in the toplevel directory called
# "AGPLv3".  If not, see <http://www.gnu.org/licenses/>.
#
"""
The mentoring XBlock was previously designed to be edited using XML.

This file contains a hack necessary for us to parse the old XML data and create blocks in Studio
from the parsed XML, as part of the upgrade process.

It works by parsing the XML and creating XBlocks in a temporary runtime
environment, so that the blocks' fields can be read and copied into Studio.
"""
import six
from xblock.fields import Scope
from xblock.runtime import (DictKeyValueStore, KvsFieldData, MemoryIdManager,
                            Runtime, ScopeIds)


class TransientRuntime(Runtime):
    """
    An XBlock runtime designed to have no persistence and no ability to render views/handlers.
    """
    def __init__(self):
        id_manager = MemoryIdManager()
        field_data = KvsFieldData(DictKeyValueStore())
        super(TransientRuntime, self).__init__(
            id_reader=id_manager,
            id_generator=id_manager,
            field_data=field_data,
        )

    def create_block_from_node(self, node):
        """
        Parse an XML node representing an XBlock (and children), and return the XBlock.
        """
        block_type = node.tag
        def_id = self.id_generator.create_definition(block_type)
        usage_id = self.id_generator.create_usage(def_id)
        keys = ScopeIds(None, block_type, def_id, usage_id)
        block_class = self.mixologist.mix(self.load_block_type(block_type))
        block = block_class.parse_xml(node, self, keys, self.id_generator)
        block.save()
        return block

    def handler_url(self, *args, **kwargs):
        raise NotImplementedError("TransientRuntime does not support handler_url.")

    def local_resource_url(self, *args, **kwargs):
        raise NotImplementedError("TransientRuntime does not support local_resource_url.")

    def publish(self, *args, **kwargs):
        raise NotImplementedError("TransientRuntime does not support publish.")

    def resource_url(self, *args, **kwargs):
        raise NotImplementedError("TransientRuntime does not support resource_url.")

    def render_template(self, *args, **kwargs):
        raise NotImplementedError("TransientRuntime cannot render templates.")


def studio_update_from_node(block, node):
    """
    Given an XBlock that is using the edX Studio runtime, replace all of block's fields and
    children with the fields and children defined by the XML node 'node'.
    """

    user_id = block.runtime.user_id
    temp_runtime = TransientRuntime()
    source_block = temp_runtime.create_block_from_node(node)

    def update_from_temp_block(real_block, temp_block):
        """
        Recursively copy all fields and children from temp_block to real_block.
        """
        # Fields:
        for field_name, field in six.iteritems(temp_block.fields):
            if field.scope in (Scope.content, Scope.settings) and field.is_set_on(temp_block):
                setattr(real_block, field_name, getattr(temp_block, field_name))
        # Children:
        if real_block.has_children:
            real_block.children = []
            for child_id in temp_block.children:
                child = temp_block.runtime.get_block(child_id)
                new_child = real_block.runtime.modulestore.create_item(
                    user_id, real_block.location.course_key, child.scope_ids.block_type
                )
                update_from_temp_block(new_child, child)
                real_block.children.append(new_child.location)
        real_block.save()
        real_block.runtime.modulestore.update_item(real_block, user_id)

    with block.runtime.modulestore.bulk_operations(block.location.course_key):
        for child_id in block.children:
            block.runtime.modulestore.delete_item(child_id, user_id)
        update_from_temp_block(block, source_block)



# -*- coding: utf-8 -*-
#
# Copyright (c) 2014-2015 Harvard, edX & OpenCraft
#
# This software's license gives you freedom; you can copy, convey,
# propagate, redistribute and/or modify this program under the terms of
# the GNU Affero General Public License (AGPL) as published by the Free
# Software Foundation (FSF), either version 3 of the License, or (at your
# option) any later version of the AGPL published by the FSF.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero
# General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program in a file in the toplevel directory called
# "AGPLv3".  If not, see <http://www.gnu.org/licenses/>.
#

import logging

import six
from xblock.core import XBlock
from xblock.fields import Scope, String
from xblock.fragment import Fragment
from xblockutils.resources import ResourceLoader
from xblockutils.studio_editable import (NestedXBlockSpec,
                                         StudioContainerWithNestedXBlocksMixin,
                                         StudioEditableXBlockMixin,
                                         XBlockWithPreviewMixin)

from .mixins import NoSettingsMixin, XBlockWithTranslationServiceMixin
from .utils import I18NService

log = logging.getLogger(__name__)
loader = ResourceLoader(__name__)


# Make '_' a no-op so we can scrape strings
def _(text):
    return text


@XBlock.needs("i18n")
class ConditionalMessageBlock(
    StudioEditableXBlockMixin, XBlockWithTranslationServiceMixin, XBlockWithPreviewMixin, XBlock
):
    """
    A message shown as part of a Step Builder review step, but only under certain conditions.
    """
    CATEGORY = 'sb-conditional-message'
    STUDIO_LABEL = _("Conditional Message")

    content = String(
        display_name=_("Message"),
        help=_("Message to display upon completion"),
        scope=Scope.content,
        default="",
        multiline_editor="html",
        resettable_editor=False,
    )

    SCORE_PERFECT, SCORE_IMPERFECT, SCORE_ANY = "perfect", "imperfect", "any"
    SCORE_CONDITIONS_DESCRIPTIONS = {
        SCORE_PERFECT: _("Show only if student got a perfect score"),
        SCORE_IMPERFECT: _("Show only if student got at least one question wrong"),
        SCORE_ANY: _("Show for any score"),
    }
    score_condition = String(
        display_name=_("Score condition"),
        default=SCORE_ANY,
        values=[{"display_name": val, "value": key} for key, val in SCORE_CONDITIONS_DESCRIPTIONS.items()],
    )

    IF_ATTEMPTS_REMAIN, IF_NO_ATTEMPTS_REMAIN, ATTEMPTS_ANY = "can_try_again", "cannot_try_again", "any"
    NUM_ATTEMPTS_COND_DESCRIPTIONS = {
        IF_ATTEMPTS_REMAIN: _("Show only if student can try again"),
        IF_NO_ATTEMPTS_REMAIN: _("Show only if student has used up all attempts"),
        ATTEMPTS_ANY: _("Show whether student can try again or not"),
    }
    num_attempts_condition = String(
        display_name=_("Try again condition"),
        default=ATTEMPTS_ANY,
        values=[{"display_name": val, "value": key} for key, val in NUM_ATTEMPTS_COND_DESCRIPTIONS.items()],
    )

    editable_fields = ('content', 'score_condition', 'num_attempts_condition')
    has_author_view = True  # Without this flag, studio will use student_view on newly-added blocks :/

    @property
    def display_name_with_default(self):
        return self._(self.STUDIO_LABEL)

    def is_applicable(self, context):
        """ Return true if this block should appear in the review step, false otherwise """
        score_summary = context['score_summary']
        attempts_remain = not score_summary['max_attempts_reached']
        if (
            (self.num_attempts_condition == self.IF_ATTEMPTS_REMAIN and not attempts_remain) or
            (self.num_attempts_condition == self.IF_NO_ATTEMPTS_REMAIN and attempts_remain)
        ):
            return False

        perfect_score = (score_summary['incorrect_answers'] == 0 and score_summary['partially_correct_answers'] == 0)
        if (
            (self.score_condition == self.SCORE_PERFECT and not perfect_score) or
            (self.score_condition == self.SCORE_IMPERFECT and perfect_score)
        ):
            return False

        return True

    def student_view_data(self, context=None):
        return {
            'block_id': six.text_type(self.scope_ids.usage_id),
            'display_name': self.display_name_with_default,
            'type': self.CATEGORY,
            'content': self.content,
            'score_condition': self.score_condition,
            'num_attempts_condition': self.num_attempts_condition,
        }

    def student_view(self, _context=None):
        """ Render this message. """
        html = u'<div class="review-conditional-message">{content}</div>'.format(
            content=self.content
        )
        return Fragment(html)

    embedded_student_view = student_view

    def author_view(self, context=None):
        fragment = self.student_view(context)
        desc = ""
        if self.num_attempts_condition == self.ATTEMPTS_ANY and self.score_condition == self.SCORE_ANY:
            desc = self._("Always shown")
        else:
            if self.score_condition != self.SCORE_ANY:
                desc += self.SCORE_CONDITIONS_DESCRIPTIONS[self.score_condition] + "<br>"
            if self.num_attempts_condition != self.ATTEMPTS_ANY:
                desc += self.NUM_ATTEMPTS_COND_DESCRIPTIONS[self.num_attempts_condition]
        fragment.content = u'<div class="conditional-message-help"><p>{}</p></div>'.format(desc) + fragment.content
        return fragment


@XBlock.needs("i18n")
class ScoreSummaryBlock(XBlockWithTranslationServiceMixin, XBlockWithPreviewMixin,
                        NoSettingsMixin, XBlock, I18NService):
    """
    Summarize the score that the student earned.
    """
    CATEGORY = 'sb-review-score'
    STUDIO_LABEL = _("Score Summary")
    has_author_view = True  # Without this flag, studio will use student_view on newly-added blocks :/

    @property
    def display_name_with_default(self):
        return self._(self.STUDIO_LABEL)

    def student_view(self, context=None):
        """ Render the score summary message. """
        context = context or {}
        html = loader.render_django_template("templates/html/sb-review-score.html",
                                             context.get("score_summary", {}),
                                             i18n_service=self.i18n_service)
        return Fragment(html)

    def student_view_data(self, context=None):
        return {
            'block_id': six.text_type(self.scope_ids.usage_id),
            'display_name': self.display_name_with_default,
            'type': self.CATEGORY,
        }

    embedded_student_view = student_view

    def author_view(self, context=None):
        context = context or {}
        if not context.get("score_summary"):
            context["score_summary"] = {
                'score': 75,
                'correct_answers': 3,
                'incorrect_answers': 1,
                'partially_correct_answers': 0,
                'correct': [],
                'incorrect': [],
                'partial': [],
                'complete': True,
                'max_attempts_reached': False,
                'show_extended_review': False,
                'is_example': True,
            }
        return self.student_view(context)


@XBlock.needs("i18n")
class PerQuestionFeedbackBlock(XBlockWithTranslationServiceMixin, XBlockWithPreviewMixin, NoSettingsMixin, XBlock,
                               I18NService):
    """
    Display any on-assessment-review-question messages.
    These messages are defined within individual questions and are only displayed if the student
    got that particular question wrong.
    """
    CATEGORY = 'sb-review-per-question-feedback'
    STUDIO_LABEL = _("Per-Question Feedback")
    has_author_view = True  # Without this flag, studio will use student_view on newly-added blocks :/

    @property
    def display_name_with_default(self):
        return self._(self.STUDIO_LABEL)

    def student_view(self, context=None):
        """ Render the per-question feedback, if any. """
        review_tips = (context or {}).get("score_summary", {}).get("review_tips")
        if review_tips:
            html = loader.render_django_template("templates/html/sb-review-per-question-feedback.html", {
                'tips': review_tips,
            }, i18n_service=self.i18n_service)
        else:
            html = u""
        return Fragment(html)

    def student_view_data(self, context=None):
        return {
            'block_id': six.text_type(self.scope_ids.usage_id),
            'display_name': self.display_name_with_default,
            'type': self.CATEGORY,
        }

    embedded_student_view = student_view

    def author_view(self, context=None):
        """ Show example content in Studio """
        context = context or {}
        if not context.get("per_question_review_tips"):
            example = self._("(Example tip:) Since you got Question 1 wrong, review Chapter 12 of your textbook.")
            context["score_summary"] = {"review_tips": [example]}
        return self.student_view(context)


@XBlock.needs("i18n")
class ReviewStepBlock(
    StudioContainerWithNestedXBlocksMixin,
    XBlockWithTranslationServiceMixin,
    XBlockWithPreviewMixin,
    NoSettingsMixin,
    XBlock
):
    """
    A dedicated step for reviewing results as the last step of a Step Builder sequence.
    """
    CATEGORY = 'sb-review-step'
    STUDIO_LABEL = _("Review Step")

    display_name = String(
        default="Review Step"
    )

    @property
    def allowed_nested_blocks(self):
        """
        Returns a list of allowed nested XBlocks. Each item can be either
        * An XBlock class
        * A NestedXBlockSpec

        If XBlock class is used it is assumed that this XBlock is enabled and allows multiple instances.
        NestedXBlockSpec allows explicitly setting disabled/enabled state,
        disabled reason (if any) and single/multiple instances.
        """
        return [
            ConditionalMessageBlock,
            NestedXBlockSpec(None, category='html', label=self._("HTML")),
            NestedXBlockSpec(ScoreSummaryBlock, single_instance=True),
            NestedXBlockSpec(PerQuestionFeedbackBlock, single_instance=True),
        ]

    def student_view(self, context=None):
        """
        Normal view of the review step.

        The parent Step Builder block should pass in appropriate context information:
        - score_summary
        """
        context = context.copy() if context else {}
        fragment = Fragment()

        if "score_summary" not in context:
            fragment.add_content(u"Error: This block only works inside a Step Builder block.")
        elif not context["score_summary"]:
            # Note: The following text should never be seen (in theory) so does not need to be translated.
            fragment.add_content(u"Your score and review messages will appear here.")
        else:
            for child_id in self.children:
                child = self.runtime.get_block(child_id)
                if child is None:  # child should not be None but it can happen due to bugs or permission issues
                    fragment.add_content(u"<p>[{}]</p>".format(u"Error: Unable to load child component."))
                else:
                    if hasattr(child, 'is_applicable'):
                        if not child.is_applicable(context):
                            continue  # Hide conditional messages that don't meet their criteria
                    # Render children as "embedded_student_view" rather than "student_view" so
                    # that Studio doesn't wrap with with unwanted controls and the XBlock SDK
                    # workbench doesn't add the acid-aside to the fragment.
                    child_fragment = self._render_child_fragment(child, context, view="embedded_student_view")
                    fragment.add_frag_resources(child_fragment)
                    fragment.add_content(child_fragment.content)

        return fragment

    def student_view_data(self, context=None):
        context = context.copy() if context else {}
        components = []

        for child_id in self.children:
            child = self.runtime.get_block(child_id)
            if hasattr(child, 'student_view_data'):
                if hasattr(context, 'score_summary') and hasattr(child, 'is_applicable'):
                    if not child.is_applicable(context):
                        continue
                components.append(child.student_view_data(context))

        return {
            'block_id': six.text_type(self.scope_ids.usage_id),
            'display_name': self.display_name,
            'type': self.CATEGORY,
            'title': self.display_name,
            'components': components,
        }

    mentoring_view = student_view

    def author_edit_view(self, context):
        """
        Add some HTML to the author view that allows authors to add child blocks.
        """
        fragment = super(ReviewStepBlock, self).author_edit_view(context)
        fragment.add_css_url(self.runtime.local_resource_url(self, 'public/css/problem-builder.css'))
        fragment.add_css_url(self.runtime.local_resource_url(self, 'public/css/problem-builder-edit.css'))
        fragment.add_javascript_url(self.runtime.local_resource_url(self, 'public/js/util.js'))
        fragment.add_javascript_url(self.runtime.local_resource_url(self, 'public/js/container_edit.js'))
        fragment.initialize_js('ProblemBuilderContainerEdit')
        return fragment



# -*- coding: utf-8 -*-
#
# Copyright (c) 2014-2015 Harvard, edX & OpenCraft
#
# This software's license gives you freedom; you can copy, convey,
# propagate, redistribute and/or modify this program under the terms of
# the GNU Affero General Public License (AGPL) as published by the Free
# Software Foundation (FSF), either version 3 of the License, or (at your
# option) any later version of the AGPL published by the FSF.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero
# General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program in a file in the toplevel directory called
# "AGPLv3".  If not, see <http://www.gnu.org/licenses/>.
#

from .base_test import MentoringBaseTest


class MultipleBlockTest(MentoringBaseTest):
    """
    Test that multiple Problem Builder blocks can happily co-exist on a page.
    """
    default_css_selector = 'div.vertical'

    def test_answer_blocks(self, expect_answer=False):
        """
        Make sure that the JavaScript is working fine even though there are many blocks on the
        page. In this test we check the answer blocks.
        """
        vertical = self.go_to_page("Multiple Problem Builders")
        blocks = vertical.find_elements_by_css_selector('.mentoring')

        block_a = blocks[0]
        self.assertIn("First", block_a.text)
        block_b = blocks[1]
        self.assertIn("Recap", block_b.text)

        answer_input = block_a.find_element_by_css_selector("textarea")
        answer_output = block_b.find_element_by_css_selector("blockquote")

        if not expect_answer:
            self.assertEqual(answer_input.text, "")
            self.assertEqual(answer_output.text, "No answer yet.")
            answer_input.send_keys('Hello there')
            submit = block_a.find_element_by_css_selector('.submit input.input-main')
            self.assertTrue(submit.is_enabled())
            submit.click()
            self.wait_until_disabled(submit)
            self.test_answer_blocks(expect_answer=True)
        else:
            self.assertEqual(answer_input.text, 'Hello there')
            self.assertEqual(answer_output.text, 'Hello there')

    def test_mcq_mrq(self):
        """
        Make sure that the JavaScript is working fine even though there are many blocks on the
        page. In this test we check the MCQ and MRQ.
        """
        vertical = self.go_to_page("Multiple Problem Builders")
        blocks = vertical.find_elements_by_css_selector('.mentoring')

        block_c = blocks[2]
        self.assertIn("Third", block_c.text)
        block_d = blocks[3]
        self.assertIn("Fourth", block_d.text)

        # Ensure that both of the blocks C and D are both working simultaneously:
        for block in (block_c, block_d):
            submit = block.find_element_by_css_selector('.submit input.input-main')
            self.assertFalse(submit.is_enabled())
            block.find_elements_by_css_selector('.choices .choice label')[0].click()

            self.assertTrue(submit.is_enabled())
            submit.click()
            self.wait_until_disabled(submit)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.25 on 2019-11-28 23:25
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('problem_builder', '0006_remove_deprecated_course_id'),
    ]

    operations = [
        migrations.AlterField(
            model_name='answer',
            name='student_id',
            field=models.CharField(db_index=True, max_length=50),
        ),
    ]



# -*- coding: utf-8 -*-
from django.db.utils import DatabaseError

from south.db import db
from south.v2 import DataMigration


class Migration(DataMigration):

    def forwards(self, orm):
        """
        Copy student data from old table to the new one.

        Problem Builder stores student answers in 'problem_builder_answer'.
        However earlier versions [may have] used 'mentoring_answer'.
        If a 'mentoring' app is currently installed on this instance, copy the student data over
        to the new table in case it is being used.
        """
        try:
            db.execute(
                'INSERT INTO problem_builder_answer ('
                'name, student_id, course_id, student_input, created_on, modified_on '
                ') SELECT '
                'name, student_id, course_id, student_input, created_on, modified_on '
                'FROM mentoring_answer'
            )
        except DatabaseError:  # Would like to just catch 'Table does not exist' but can't do that in a db-agnostic way
            print(" - Seems like mentoring_answer does not exist. No data migration needed.")

    def backwards(self, orm):
        raise RuntimeError("Cannot safely reverse this migration.")

    models = {
        'problem_builder.answer': {
            'Meta': {'unique_together': "(('student_id', 'course_id', 'name'),)", 'object_name': 'Answer'},
            'course_id': ('django.db.models.fields.CharField', [], {'max_length': '50', 'db_index': 'True'}),
            'created_on': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'modified_on': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50', 'db_index': 'True'}),
            'student_id': ('django.db.models.fields.CharField', [], {'max_length': '32', 'db_index': 'True'}),
            'student_input': ('django.db.models.fields.TextField', [], {'default': "''", 'blank': 'True'})
        }
    }

    complete_apps = ['problem_builder']
    symmetrical = True
